Package: GPCMlasso
Type: Package
Title: Differential Item Functioning in Generalized Partial Credit
        Models
Version: 0.1-9
Date: 2026-05-11
Authors@R: person(given = "Gunther",
                      family = "Schauberger",
                      role = c("aut", "cre"),
                      email = "gunther.schauberger@tum.de")
Maintainer: Gunther Schauberger <gunther.schauberger@tum.de>
Description: Provides a framework to detect Differential Item Functioning (DIF) in Generalized Partial Credit Models (GPCM) and special cases of the GPCM as proposed by Schauberger and Mair (2019) <doi:10.3758/s13428-019-01224-2>. A joint model is set up where DIF is explicitly parametrized and penalized likelihood estimation is used for parameter selection. The big advantage of the method called GPCMlasso is that several variables can be treated simultaneously and that both continuous and categorical variables can be used to detect DIF.
License: GPL (>= 2)
Imports: Rcpp (>= 0.12.4), TeachingDemos, cubature, caret, statmod,
        mvtnorm, mirt, methods
Depends: ltm
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-11 05:04:34 UTC; ge29weh
Author: Gunther Schauberger [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-11 06:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-11 08:38:46 UTC; unix
Archs: GPCMlasso.so.dSYM

Package: GPCMlasso
Type: Package
Title: Differential Item Functioning in Generalized Partial Credit
        Models
Version: 0.1-9
Date: 2026-05-11
Authors@R: person(given = "Gunther",
                      family = "Schauberger",
                      role = c("aut", "cre"),
                      email = "gunther.schauberger@tum.de")
Maintainer: Gunther Schauberger <gunther.schauberger@tum.de>
Description: Provides a framework to detect Differential Item Functioning (DIF) in Generalized Partial Credit Models (GPCM) and special cases of the GPCM as proposed by Schauberger and Mair (2019) <doi:10.3758/s13428-019-01224-2>. A joint model is set up where DIF is explicitly parametrized and penalized likelihood estimation is used for parameter selection. The big advantage of the method called GPCMlasso is that several variables can be treated simultaneously and that both continuous and categorical variables can be used to detect DIF.
License: GPL (>= 2)
Imports: Rcpp (>= 0.12.4), TeachingDemos, cubature, caret, statmod,
        mvtnorm, mirt, methods
Depends: ltm
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-11 05:04:34 UTC; ge29weh
Author: Gunther Schauberger [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-11 06:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-11 08:03:04 UTC; unix
Archs: GPCMlasso.so.dSYM

useDynLib(GPCMlasso, .registration = TRUE)

importFrom(Rcpp, evalCpp)
importFrom(TeachingDemos, spread.labs)
importFrom(statmod, gauss.quad)
importFrom(cubature, pcubature)
import(ltm)
importFrom(mirt,mirt)
importFrom(caret, class2ind)
importFrom(mvtnorm, rmvnorm)


importFrom("utils", "combn", "head", "tail")
importFrom("stats", "dnorm", "coef", "model.matrix", "model.frame",
           "model.response", "optim", "qnorm","sd", "integrate","nlminb", "nlm","rbinom", "rmultinom", "rnorm", "terms")
importFrom("graphics", "abline", "lines", "plot","par", "segments", "strheight", "text")
  importFrom("methods", "selectMethod")
importFrom(parallel, makeCluster,clusterExport,parLapply,stopCluster,parSapply, clusterCall)
  
export(GPCMlasso, ctrl_GPCMlasso, trait.posterior)
S3method(plot, GPCMlasso)
S3method(print, GPCMlasso)
S3method(predict, GPCMlasso)

#' Plot function for GPCMlasso
#' 
#' Plot function for a \code{GPCMlasso} object. Plots show coefficient paths
#' of DIF (or DSF) parameters along (a transformation of) the tuning parameter lambda.
#' One plot per item is created, every single parameter corresponding to this item
#' is depicted by a single path. 
#' The optimal model is highlighted with a red dashed line. 
#' 
#' @usage \method{plot}{GPCMlasso}(x, select = c("BIC", "AIC", "cAIC", "cv"), 
#' type = c("DIF", "Variable"),
#' log.lambda = TRUE, items_per_page = 1, items = "all", 
#' columns = NULL, ask_new = TRUE, lambda.lines = TRUE,
#' equal_range = TRUE, add.to.lambda = 0, \dots)
#' @param x \code{GPCMlasso} object
#' @param select Specifies which criterion to use for the optimal model, we recommend the 
#' default value "BIC". If cross-validation was performed, automatically the optimal
#' model according to cross-validation is used. The chosen optimal model is 
#' highlighted with a red dashed line. 
#' @param type Which plot type should be used. \code{DIF} means that one plot per item is shown, 
#' indicating whether DIF was found for this item. \code{Variable} means that
#' one plot per variable is shown, which is useful for the option \code{fuse.per.variable}. 
#' @param log.lambda A logical value indicating whether lambda or a log-transformation of 
#' lambda should be used as x-axis in the plots.
#' @param items_per_page By default, each plot/item is put on a separate page. For example,
#' \code{items_per_page=4} would put four plots/items on one page.
#' @param items By default, all items are plotted. If \code{items=c(1,3)}, 
#' only the first and the third item are plotted.
#' @param columns Specifies the number of columns to use when several
#' plots are on one page. Only relevant if \code{items_per_page}>1. 
#' @param ask_new If TRUE, the user is asked to confirm before the next item is plotted.
#' @param lambda.lines A logical value indicating whether a thin gray line plotted
#' for each value from the vector of tuning parameters from \code{object}
#' @param equal_range A logical value indicating whether for each plot equal limits 
#' on the y-axis shall be used.
#' @param add.to.lambda Constant c to be added to lambda as in log(lambda + c) for plotting coefficient paths.
#' @param ... Further plot arguments.
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @seealso \code{\link{GPCMlasso}}
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
plot.GPCMlasso <- function(x, select = c("BIC", "AIC", "cAIC", "cv"), 
                           type = c("DIF", "Variable"),
                           log.lambda = TRUE, items_per_page = 1, items = "all", 
                           columns = NULL, ask_new = TRUE, lambda.lines = TRUE,
                           equal_range = TRUE, add.to.lambda = 0, ...){
  op <- par(no.readonly = TRUE)
# browser()
  select <- match.arg(select, c("BIC", "AIC", "cAIC", "cv"))
  type <- match.arg(type, c("DIF", "Variable"))
  
  if(select=="BIC"){
    criterion <- x$BIC
  }
  if(select=="AIC"){
    criterion <- x$AIC
  }
  if(select=="cAIC"){
    criterion <- x$cAIC
  }
  if(select=="cv" | !is.null(x$cv_error)){
    criterion <- x$cv_error
    if(is.null(x$cv_error)){
      warning("No cross-validation was performed for the model. Instead of the cross-validation error the optimal model is selected by BIC")
      criterion <- x$BIC
    }
  }
  ###############
  ## Start of first (original) plot type
  ###############
    if(type == "DIF"){  
  with(x$design_list,{
    if(m==0){
      stop("No covariates, nothing to plot!")
    }
    if(length(x$control$lambda)==1){
      stop("Only one tuning parameter, nothing to plot!")
    }

# browser()
      
    n.dif.par <- I*m
    if(x$DSF){
      n.dif.par <- sum(q)*m
    }
    
    if(identical(items, "all")){items <- 1:I}
    
    n.plots <- length(items)
    pages <- ceiling(n.plots/items_per_page)
    
    coefs <- x$coefficients
    
    gamma.start <- sum(q)+1
    if(RSM){
      gamma.start <- q[1]+I
    }
    
    if(x$main.effects){
      gamma.start <- gamma.start + m
    }
    
    gamma <- coefs[,gamma.start:(gamma.start+n.dif.par-1)]
    
    
    if (is.null(columns)) {
      cols <- floor(sqrt(items_per_page))
    } else {
      cols <- columns
    }
    rows <- ceiling(items_per_page/cols)
    
    g.range = range(gamma, na.rm = TRUE)
    
    start.gamma <-1
    plots_on_page <- 0
    pages_done <- 0
    par(mfrow=c(rows,cols),xpd=TRUE)
    
    for(u in 1:I){
      
      
      if(x$DSF){
        par.item <- q[u]*m
      }else{
        par.item <- m
      }
      if(u %in% items){
        plothelp.item(gamma[,start.gamma:(start.gamma+par.item-1), drop = FALSE], 
                      x$item.names[u], par.item, x$control$lambda, g.range, 
                      equal_range, criterion, x.names, log.lambda,
                      lambda.lines)
        plots_on_page <- plots_on_page+1
        if(plots_on_page==items_per_page & pages_done<(pages-1)){
          plots_on_page <- 0
          pages_done <- pages_done+1
          if(interactive() & ask_new)
          {readline("Press enter for next plot!")}
          par(mfrow=c(rows,cols),xpd=TRUE)
        }
      }
      start.gamma <- start.gamma+par.item    }
    
  })
    ###############  
    ## Start of second plot type (added in Oct 2024)
    ###############
    }else{
      with(x$design_list,{
      if(m==0){
        stop("No covariates, nothing to plot!")
      }
      if(length(x$control$lambda)==1){
        stop("Only one tuning parameter, nothing to plot!")
      }
      # browser()
      
      
      n.dif.par <- I*m
      if(x$DSF){
        n.dif.par <- sum(q)*m
      }
      
      if(identical(items, "all")){items <- 1:I}
      
      n.plots <- length(items)
      pages <- ceiling(n.plots/items_per_page)
      
      coefs <- x$coefficients
      
      gamma.start <- sum(q)+1
      if(RSM){
        gamma.start <- q[1]+I
      }
      
      if(x$main.effects){
        gamma.start <- gamma.start + m
      }
      
      gamma <- coefs[,gamma.start:(gamma.start+n.dif.par-1)]
      
      coef.list <- list()
      
      for(i in 1:nrow(x$coefficients)){
        
        coef <- x$coefficients[i,]
        
        coef.main <- coef[(gamma.start-m):(gamma.start-1)]
        
        coef.dif <- coef[gamma.start:(gamma.start+n.dif.par-1)]
        
        coef.sum <- matrix(coef.dif,ncol=I)
        colnames(coef.sum) <- x$item.names
        rownames(coef.sum) <- x.names
        
        
        coef.sum <- coef.sum + matrix(rep(coef.main,each=length(x$item.names)),ncol = I,byrow=TRUE)
        
        coef.list[[i]] <- coef.sum
      }
      
      list.var <- list()
      for(i in 1:length(x.names)){
        var.mat <- c()
        for(j in 1:length(x$control$lambda)){
          var.mat <- rbind(var.mat, coef.list[[j]][i,])
        }
        list.var[[i]] <- var.mat
      }
      
      if (is.null(columns)) {
        cols <- floor(sqrt(items_per_page))
      } else {
        cols <- columns
      }
      rows <- ceiling(items_per_page/cols)
      
      g.range = range(gamma, na.rm = TRUE)
      
      start.gamma <-1
      plots_on_page <- 0
      pages_done <- 0
      par(mfrow=c(rows,cols),xpd=TRUE)
      
      
      
      for(i in 1:m){
      
        
        if(log.lambda){
          xlab <- bquote(log(lambda+.(add.to.lambda)))
          lambda <- log(x$control$lambda+add.to.lambda)
          if(add.to.lambda == 0){
            xlab <- bquote(log(lambda))

          }
        }else{
          xlab <- expression(lambda)
          lambda <- x$control$lambda
        }
        xlim <- rev(range(lambda))
                
        coef.i <- list.var[[i]]  
        
        i.lim <- range(coef.i, na.rm = TRUE)

        plot(lambda, coef.i[,1], type="l",main=x.names[i],ylim= range(coef.i),xlim = xlim,
             ylab ="coefficients", xlab = xlab,frame.plot=FALSE)
        for(j in 2:I){
          lines(lambda, coef.i[,j])
        }
        

        
        x.lab1 <- min(lambda)-abs(diff(range(lambda)))*0.02
        x.lab2 <- min(lambda)-abs(diff(range(lambda)))*0.005
        y.lab1 <- coef.i[length(lambda),]
        y.lab2 <- spread.labs(y.lab1, 1.2*strheight("A"))
        text( x.lab1, y.lab2, x$item.names,pos=4)
        segments( x.lab2, y.lab1,
                  x.lab1, y.lab2 ,col="gray")
        
        
        if(lambda.lines){
          segments(x0=lambda,y1=i.lim[1],y0=i.lim[2],lwd=.5, lty=2, col="lightgray")
          # abline(v=lambda, lwd=.5, lty=2, col="lightgray")
        }
        
        if(!is.null(criterion)){
          segments( lambda[which.min(criterion)], max(i.lim),
                    lambda[which.min(criterion)], min(i.lim) ,
                    col=2,lty=2,lwd=par()$lwd)
        }
        
        plots_on_page <- plots_on_page+1
        if(plots_on_page==items_per_page & pages_done<(pages-1)){
          plots_on_page <- 0
          pages_done <- pages_done+1
          if(interactive() & ask_new)
          {readline("Press enter for next plot!")}
          par(mfrow=c(rows,cols),xpd=TRUE)
        }
      }
      })
    }
  
  par(op)
  invisible(x)
}

plothelp.item <- function(item, name, par.item, lambda, g.range, equal_range, 
                          criterion,  x.names, log.lambda, lambda.lines){
  # browser()
  if(equal_range){
    i.lim <- g.range
  }else{
    i.lim <- range(item, na.rm = TRUE)
  }
  
  if(log.lambda){
    xlab <- expression(log(lambda+1))
    lambda <- log(lambda+1)
  }else{
    xlab <- expression(lambda)
  }
  
  x.lim <- rev(range(c(lambda,min(lambda)-abs(diff(range(lambda)))*0.1)))
  
  plot(lambda, item[,1],ylim=i.lim,type="l", main=name,xlim=x.lim,
       xlab=xlab,ylab="parameters",frame.plot=FALSE,
       lwd=par()$lwd)
  if(par.item>1){
    for(uu in 2:par.item){
      lines(lambda, item[,uu],lwd=par()$lwd)
    }
  }
  if(lambda.lines){
    segments(x0=lambda,y1=i.lim[1],y0=i.lim[2],lwd=.5, lty=2, col="lightgray")
    # abline(v=lambda, lwd=.5, lty=2, col="lightgray")
  }
  
  
  if(!is.null(criterion)){
    segments( lambda[which.min(criterion)], max(i.lim),
              lambda[which.min(criterion)], min(i.lim) ,
              col=2,lty=2,lwd=par()$lwd)
  }
  
  x.lab1 <- min(lambda)-abs(diff(range(lambda)))*0.02
  x.lab2 <- min(lambda)-abs(diff(range(lambda)))*0.005
  y.lab1 <- item[length(lambda),]
  y.lab2 <- spread.labs(y.lab1, 1.2*strheight("A"))
  
  ncat <- par.item/length(x.names)
  if(ncat>1){
    x.names <- paste(rep(x.names,ncat),rep(1:ncat,each=length(x.names)),sep=".")
  }
  
  text( x.lab1, y.lab2, x.names,pos=4)
  segments( x.lab2, y.lab1,
            x.lab1, y.lab2 ,col="gray")
  
  
  ## used for illustrating plot in paper
  # segments( 3, i.lim[1],
  #           3, i.lim[2] ,col="gray",lty=2, lwd = 2)
  
}

fit_GPCMlasso <- function(model = model, loglik_fun = loglikPCMlasso, score_fun = scorePCMlasso,
                          log_score_fun = loglikscorePCMlasso,
                          design_list = design_list, 
                          control = control, start = NULL, scale_fac = 1, main.effects = main.effects){

  ## start with-expression to use everything from design_list and control
  with(c(design_list,control),{
  
    ## get nodes and weights for Gauss-Hermite quadrature
    her_poly <- gauss.quad(Q, "hermite")
    GHnodes <- her_poly$nodes
    GHweights <- her_poly$weights * exp(GHnodes^2) * dnorm(GHnodes)

  ## lower bounds for variance parameters
  l.bound <- c(rep(-Inf,px-n_sigma), rep(0,n_sigma))

  if(!is.null(lambda)){
    l.lambda <- NA
  }  

  ## get values for penalty weight, start values and lambda grid    
  help_me <- help_fit(model, Y, l.lambda, start, loglik_fun, score_fun, log_score_fun, adaptive,
                      Q,q,I,n,m,response,design,designX, px,
                      GHweights, GHnodes, acoefs, lambda2, cvalue, n_sigma,
                      l.bound, trace, log.lambda, weight.penalties, scale_fac, ada.lambda,
                      lambda.min, ada.power, cores, null_thresh, DSF, gradtol, iterlim, steptol,
                      main.effects = main.effects, penalize.main.effects = penalize.main.effects,
                      ctrl.gpcm = ctrl.gpcm)
  
  weight <- help_me$weight
  alpha.start <- help_me$alpha.start
  if(is.null(lambda)){
    lambda <- help_me$lambda
  }
  
  ## start estimation along lambda
  coef.final <- coef.orig <- matrix(0, nrow=length(lambda), ncol = px)
  logLik <- df <- c()

  ########################
  for(l in seq_along(lambda)){
    if(trace){
    cat(paste0(l,". lambda out of ", length(lambda),":"), lambda[l],"\n")
    }
suppressWarnings(
    m.opt <- try(nlm(log_score_fun, alpha.start,
                        Q = Q, q = q, I = I, n = n, Y = response, X = design, Z = designX, px = px,
                        GHweights = GHweights, GHnodes = GHnodes,
                        acoefs = acoefs, lambda = lambda[l], scale_fac = scale_fac,
                        lambda2 = lambda2, cvalue = cvalue, cores = cores, weight = weight,  n_sigma = n_sigma,
                         gradtol = gradtol, iterlim = iterlim, check.analyticals = FALSE, 
                        steptol = steptol)))

    coefs.l <- try(m.opt$estimate)

    if(inherits(m.opt, "try-error")){
      browser()
        m.opt <- try(nlminb(start = alpha.start, objective = loglik_fun, gradient = score_fun,
               Q = Q, q = q, I = I, n = n, Y = response, X = design, Z = designX, px = px,
               GHweights = GHweights, GHnodes = GHnodes,
               acoefs = acoefs, lambda = lambda[l], scale_fac = scale_fac,
               lambda2 = lambda2, cvalue = cvalue, cores = cores, weight = weight,  n_sigma = n_sigma,
               control=list(eval.max=500, iter.max=500, step.min=0.01),
               lower=l.bound))
        coefs.l <- m.opt$par
    }
    alpha.start <- coefs.l
      # browser()
    coefs.l[abs(coefs.l)<null_thresh & rowSums(abs(acoefs))!=0] <- 0
      
    coef.orig[l,] <- coefs.l
    
      coefs.l <- round(coefs.l, precision)
      
      n.pen.help <- m*I
      if(main.effects & penalize.main.effects){
        n.pen.help <- n.pen.help+m
        }

      if(sum(colSums(abs(acoefs))!=0)==n.pen.help){
        df.l <- sum(abs(t(acoefs)%*%coefs.l)!=0)
      }else{
        if(fuse.per.variable){
          ## needs to be modified
          df.l <- 10
        }else{
        df.l <- 0
        coefs.pen <- coefs.l[rowSums(abs(acoefs))!=0]
        start.coefs <- 1
        if(main.effects & penalize.main.effects){
          start.coefs <- m+1
          df.l <- df.l + sum(coefs.pen[1:m]!=0)
        }
        for(i in 1:I){
          coefs.i <- coefs.pen[start.coefs:(start.coefs-1+q[i]*m)]
          gamma.index <- matrix(1:(q[i]*m),nrow=m)
          for(ii in 1:m){
            coefs.ii <- coefs.i[gamma.index[ii,]]
            df.l <- df.l + sum(unique(coefs.ii)!=0)
          }
          start.coefs <- q[i]*m+start.coefs
        }

        }
      }
      
      df.l <- df.l + sum(rowSums(abs(acoefs))==0)
      
      logLik.l <- -loglik_fun(coefs.l, Q = Q, q = q, I = I, n = n,
                             Y = response, X = design, Z = designX, px = px,
                             GHweights = GHweights, GHnodes = GHnodes, 
                             acoefs = acoefs, lambda = 0, lambda2 = 0, cvalue = cvalue, 
                             cores = cores, weight = weight,  n_sigma = n_sigma, scale_fac = 1)

      coef.final[l,] <- coefs.l
      logLik[l] <- logLik.l
      df[l] <- df.l
  }

  coef.rescal <- round(t(t(coef.orig)/sd.vec), precision)
  
  
  ret.list <- list(coefficients = coef.final, coef.rescal = coef.rescal,
                   logLik = logLik, lambda = lambda, df = df)
  
  return(ret.list)
  
  ## end of with-expression
  })

}

fit_cv_GPCMlasso <- function(model, design_list,
                             control, score_fun, loglik_fun, log_score_fun, Y,
                             main.effects){
  
  with(design_list,{

        ## create cv folds
    fold_list <- get_fold_list(n=n, folds = control$folds)
    
    cat("Cross-Validation...", "\n")

    cv_result <- sapply(seq(control$folds), cv_fun, fold_list, Y,
                      control,  model, design_list,
                      loglik_fun, score_fun, log_score_fun,
                      main.effects)

   cv_error <- rowSums(cv_result)/design_list$n

  return(cv_error)  
    
  })
}


cv_fun <- function(ff, fold_list, Y, control, model,
                   design_list, loglik_fun, score_fun, log_score_fun, main.effects) {
# browser()
  I <- design_list$I
  n <- design_list$n
  n_sigma <- design_list$n_sigma
  q <- design_list$q
  
  scale_fac <- 1-1/control$folds
  
  if (control$trace) {
    cat("CV-fold:", ff, "out of", control$folds, "\n")
  }

  index <- matrix(1:(n*sum(q)),nrow=sum(q))
  index.test <- c(index[,fold_list[[ff]]])
  index.train <- c(index[,-fold_list[[ff]]])
  
  design.test <- design_list$designX[index.test,]
  design_list$designX <- design_list$designX[index.train,]
  
  response.test <- design_list$response[index.test]
  design_list$response <- design_list$response[index.train]
  
  design_list$n <- length(index.train)/sum(q)
  
  ## get fitted parameters 
  fit.fold <- fit_GPCMlasso(model = model, loglik_fun = loglik_fun, score_fun = score_fun, 
                            log_score_fun = log_score_fun,
                            design_list = design_list, 
                            control = control, start = NULL, scale_fac = scale_fac, main.effects = main.effects)
  coef.fold <- fit.fold$coefficients
  px <- ncol(coef.fold)
  
  criterion <- c()
  
  ## get nodes and weights for Gauss-Hermite quadrature
  her_poly <- gauss.quad(control$Q, "hermite")
  GHnodes <- her_poly$nodes
  GHweights <- her_poly$weights * exp(GHnodes^2) * dnorm(GHnodes)
  

  
  for(e in 1:nrow(coef.fold)){
  criterion[e] <-  loglik_fun(coef.fold[e,],
                     response.test,
                     design_list$design,
                     design.test,
                     control$Q,
                     q, length(fold_list[[ff]]), I, px,
                     GHweights,
                     GHnodes,
                     design_list$acoefs,
                     0,
                     0,
                     control$cvalue,
                     control$cores,
                     rep(1,ncol(design_list$acoefs)),
                     n_sigma,1)
  }
  criterion
}



get_fold_list <- function(n, folds){
  
  n.cv <- rep(floor(n/folds), folds)
  rest <- n%%folds
  if (rest > 0) {
    n.cv[1:rest] <- n.cv[1:rest] + 1
  }
  
  remaining_persons <- 1:n
  fold_list <- list()
  
  for(o in 1:folds){
    sample.o <- sample(remaining_persons[!is.na(remaining_persons)], n.cv[o])
    fold_list[[o]] <- sample.o
    remaining_persons[sample.o] <- NA
  }
  
  fold_list
}

start_values <- function(RSM, GPCM, Y, q, p_pen,p, n_sigma, person_list){
  Y2 <- matrix(NA, ncol=ncol(Y), nrow=nrow(Y))
  for(o in 1:length(person_list)){
    Y2[o,person_list[[o]]+1] <- Y[o,person_list[[o]]+1]
  }
  
  if(!RSM ){
    if(q>1){
      if(GPCM){
        m.ltm <- gpcm(Y2, constraint = "gpcm")
        coef.ltm <- coef(m.ltm)
        sigma.start <- coef.ltm[,q+1]
        coef.ltm <- coef.ltm*sign(sigma.start)
      }else{
        m.ltm <- gpcm(Y2, constraint = "1PL")
        coef.ltm <- coef(m.ltm)
        sigma.start <- coef.ltm[1,q+1]
        coef.ltm <- coef.ltm*sign(sigma.start)
      }
    }else{
      if(GPCM){
        m.ltm <- gpcm(Y2, constraint = "gpcm")
        coef.ltm <- -coef(m.ltm)
        sigma.start <- coef.ltm[,q+1]
        coef.ltm <- coef.ltm*sign(sigma.start)
      }else{
        m.ltm <- rasch(Y2)
        coef.ltm <- -coef(m.ltm)
        sigma.start <- coef.ltm[1,q+1]
        coef.ltm <- coef.ltm*sign(sigma.start)
      } 
    }
    delta.start <- c(t(coef.ltm[,-(q+1)]))
    alpha.start <- c(delta.start,rep(0,p_pen),abs(sigma.start))
  }else{
    alpha.start <- c(rep(0, p-n_sigma), rep(0.1,n_sigma))
    # try(optim(par = alpha.start, fn = loglik_fun, gr = score_fun,
    #           Q = Q, q = q, I = I, n = n, Y = response, X = design, Z = designX, p = p,
    #           GHprobs = GHprobs, GHweights = GHweights, GHnodes = GHnodes,
    #           acoefs = acoefs, lambda = lambda[l],
    #           lambda2 = lambda2, con = con, weight = weight,  n_sigma = n_sigma,
    #           pers_list = person_list, scalefac = scalefac,
    #           lower = l.bound, method = "L-BFGS-B",
    #           control = list(parscale = parscale_now, factr = factr)))
    # 
  }
  
 return(alpha.start)
}

loglikscoreDIFlasso2 <- function(alpha, Y, X, Z, Q, q, n, I,
                                 px, GHweights, GHnodes,
                                 acoefs, lambda, lambda2, cvalue, cores,
                                 weight, n_sigma, scale_fac){
  l <- loglikscoreDIFlasso(alpha, Y, X, Z, Q, q, n, I,
  px, GHweights, GHnodes,
  acoefs, lambda, lambda2, cvalue, cores,
  weight, n_sigma, scale_fac)
  
  ret <- l$objective
  attr(ret,"gradient") <- l$gradient
  ret
}

loglikscorePCMlasso2 <- function(alpha, Y, X, Z, Q, q, n, I,
                                 px, GHweights, GHnodes,
                                 acoefs, lambda, lambda2, cvalue, cores,
                                 weight, n_sigma, scale_fac){
  l <- loglikscorePCMlasso(alpha, Y, X, Z, Q, q, n, I,
                           px, GHweights, GHnodes,
                           acoefs, lambda, lambda2, cvalue, cores,
                           weight, n_sigma, scale_fac)
  
  ret <- l$objective
  attr(ret,"gradient") <- l$gradient
  ret
}

help_fit <- function(model, Y, l.lambda, start, loglik_fun, score_fun, log_score_fun, adaptive,
                        Q,q,I,n,m,response,design,designX, px,
                        GHweights, GHnodes, acoefs, lambda2, cvalue, n_sigma,
                        l.bound, trace, log.lambda, weight.penalties, scale_fac = scale_fac,
                        ada.lambda, lambda.min, ada.power, cores,
                        null_thresh, DSF, gradtol,iterlim, steptol, main.effects, 
                     penalize.main.effects, ctrl.gpcm){

  ## get initial weight parameters   
  weight <- rep(1,ncol(acoefs))
# browser()
  ## initialize starting values
  if(is.null(start)){
    if(trace){
      cat("Find start values ...\n")
    }
    alpha.start <- c(rep(0.1, px))

    if(!(model %in% c("RSM","GRSM"))){
    if(model=="GPCM"){
      m.ltm <- gpcm(Y, constraint = "gpcm", control = ctrl.gpcm)
      coef.ltm <- coef(m.ltm)
      if(is.matrix(coef.ltm)){
        sigma.start <- coef.ltm[,q[1]+1]
        delta.start <- c(t(coef.ltm[,-(q[1]+1)]))
      }else{
        sigma.start <- delta.start <- c()
        for(u in 1:I){
          sigma.start[u] <- coef.ltm[[u]][q[u]+1]
          delta.start <- c(delta.start, coef.ltm[[u]][-(q[u]+1)])
        }
      }
      
    }
    if(model=="PCM"){
      m.ltm <- gpcm(Y, constraint = "1PL", control = ctrl.gpcm)
      coef.ltm <- coef(m.ltm)
      if(is.matrix(coef.ltm)){
        sigma.start <- coef.ltm[1,q[1]+1]
        delta.start <- c(t(coef.ltm[,-(q[1]+1)]))
      }else{
        delta.start <- c()
        sigma.start <- coef.ltm[[1]][q[1]+1]
        for(u in 1:I){
          delta.start <- c(delta.start, coef.ltm[[u]][-(q[u]+1)])
        }
      }
    }
    if(model=="2PL"){
      m.ltm <- gpcm(Y, constraint = "gpcm", control = ctrl.gpcm)
      coef.ltm <- coef(m.ltm)
      sigma.start <- coef.ltm[,q[1]+1]
      delta.start <- c(t(coef.ltm[,-(q[1]+1)]))
    }
    if(model=="RM"){
      m.ltm <- gpcm(Y, constraint = "1PL", control = ctrl.gpcm)
      coef.ltm <- coef(m.ltm)
      sigma.start <- coef.ltm[1,q[1]+1]
      delta.start <- c(t(coef.ltm[,-(q[1]+1)]))
    }
    
    }else{

    if(model=="RSM"){
      m.mirt <- mirt(Y, 1, itemtype = 'rsm', verbose = FALSE)
      coefmethod <- selectMethod("coef", class(m.mirt))
      coef.mirt <- coefmethod(m.mirt, simplify = TRUE)
      sigma.start <- sqrt(coef.mirt$cov)
      alpha.mirt <- (coef.mirt$items)[1,2:(q[1]+1)]
      delta.mirt <- (coef.mirt$items)[,(q[1]+2)]
      delta.mirt <- -delta.mirt + alpha.mirt[1]
      alpha.mirt <- alpha.mirt - alpha.mirt[1]
      delta.start <- c(delta.mirt, alpha.mirt[-1])
    }
      if(model=="GRSM"){
        m.mirt <- mirt(Y, 1, itemtype = 'rsm', verbose = FALSE)
        coefmethod <- selectMethod("coef", class(m.mirt))
        coef.mirt <- coefmethod(m.mirt, simplify = TRUE)
        sigma.start <- rep(sqrt(coef.mirt$cov),n_sigma)
        alpha.mirt <- (coef.mirt$items)[1,2:(q[1]+1)]
        delta.mirt <- (coef.mirt$items)[,(q[1]+2)]
        delta.mirt <- -delta.mirt + alpha.mirt[1]
        alpha.mirt <- alpha.mirt - alpha.mirt[1]
        delta.start <- c(delta.mirt, alpha.mirt[-1])
      }
    }
    
      alpha.start <- c(delta.start,rep(0,ncol(designX)),abs(sigma.start))

      alpha.null <- alpha.start[rowSums(abs(acoefs))==0]
      p_null <- length(alpha.null)
      design_null <- matrix(0,0,0)
      if(main.effects & ncol(designX)>0 & (!penalize.main.effects)){
        design_null <- designX[,1:m, drop = FALSE]
      }
      acoefs_null <- matrix(0,nrow=p_null,ncol=1)
      bound_null <- l.bound[rowSums(abs(acoefs))==0]

      loglik_NA <- TRUE


      while(loglik_NA){
        loglik_NA <- is.nan(log_score_fun(alpha.null,
                                Q = Q, q = q, I = I, n = n, Y = response, X = design,
                                Z = design_null, px = p_null,
                                GHweights = GHweights, GHnodes = GHnodes,
                                acoefs = acoefs_null, lambda = 0, scale_fac = scale_fac,
                                lambda2 = lambda2, cvalue = cvalue, cores = cores, weight = 1,  n_sigma = n_sigma))
        if(loglik_NA){
          alpha.null <- alpha.null*0.9
        }
              
      }

      m.opt <- try(nlminb(start = alpha.null, objective = loglik_fun, gradient = score_fun,
                    Q = Q, q = q, I = I, n = n, Y = response, X = design,
                    Z = design_null, px = p_null,
                    GHweights = GHweights, GHnodes = GHnodes,
                    acoefs = acoefs_null, lambda = 0, scale_fac = scale_fac,
                    lambda2 = lambda2, cvalue = cvalue, cores = cores, weight = 1,  n_sigma = n_sigma,
                    control=list(eval.max=500, iter.max=500, step.min=0.01),
                    lower=bound_null))

      alpha.start[rowSums(abs(acoefs))==0] <- m.opt$par
      alpha.start[rowSums(abs(acoefs))!=0] <- 1e-8

  }else{
    alpha.start <- start
  }

  
  ##  get new weights if necessary
  if(adaptive){
    if(trace){
      cat("Get adaptive weights ...", "\n")
    }
 

    m.opt <- try(nlminb(start = alpha.start, objective = loglik_fun, gradient = score_fun,
                        Q = Q, q = q, I = I, n = n, Y = response, X = design,
                        Z = designX, px = px,
                        GHweights = GHweights, GHnodes = GHnodes,
                        acoefs = acoefs, lambda = 0, scale_fac = scale_fac,
                        lambda2 = ada.lambda, cvalue = cvalue, cores = cores, weight = weight,  n_sigma = n_sigma,
                        control=list(eval.max=500, iter.max=500,step.min=0.01),
                        lower=l.bound))
    
    weight <- try(m.opt$par)

    if(inherits(m.opt,"try-error")){
      stop("Adaptive weights can not be calculated! Increase ada.lambda or set adaptive = FALSE!")
    }
    weight <- abs(t(acoefs) %*% weight)^ada.power
    if (any(weight == 0)) 
      weight[which(weight == 0)] <- 1e-4
    weight <- as.vector(1/weight)
  }

  # if(weight.penalties & length(weight)!=(I*m) & m>0){
  #   weight.i <- (q+choose(q,2))/q
  #   weight.help <- c(rep(weight.i,q*m), rep(weight.i,choose(q,2)*m))
  #   weight.help <- weight.help/(prod(weight.help)^(1/length(weight.help)))
  #   weight <- weight/weight.help
  # }

  
  #### find maximal lambda value and make grid
  if(!is.na(l.lambda)){
    
    if(trace){
      cat("Find maximal tuning parameter ...", "\n")
    }
    
  score <- score_fun(alpha.start, response, design, designX, Q,
                         q, n, I, px, GHweights, GHnodes, acoefs,
                         0, lambda2, cvalue,cores,  weight,
                         n_sigma, scale_fac)

  a <- abs(score/acoefs%*%weight)
  a[a==Inf] <- 0
  lambda.max <- max(a[rowSums(abs(acoefs))!=0])*1.1

    if(DSF){
    lambda.max <- find.lambda(lambda.max, l.lambda, alpha.start, log_score_fun,
                              Q,q,I,n,response,design,designX, px, 
                              GHweights, GHnodes, acoefs, scale_fac, lambda2, cvalue, 
                              cores, weight, n_sigma, null_thresh, gradtol, iterlim, 
                              steptol)
  }

if(log.lambda){
  
  correct.factor <- 0.0
    lambda <- exp(seq(log(lambda.max+correct.factor*lambda.max), 
                      log(lambda.min+correct.factor*lambda.max), length = l.lambda))-
      correct.factor*lambda.max
    lambda[l.lambda] <- lambda.min
  }else{
    lambda <- seq(lambda.max,lambda.min,length=l.lambda)
  }
  }else{
    lambda <- NA
  }

  return(list(lambda = lambda, weight = weight, alpha.start = alpha.start))
}

#' GPCMlasso
#' 
#' Performs GPCMlasso, a method to identify differential item functioning (DIF) in Generalized Partial Credit Models. 
#' A joint parametric model is set up based on an IRT model chosen by the user. Several variables can be considered simultaneously.
#' For each pair between variable and item, a parametric DIF effect is introduced which
#' indicates DIF if the respective parameter is selected (estimated to be unequal zero). 
#' Parameter selection is done using a lasso-type penalization term.
#' 
#' @param formula Formula to indicate which items are considered and which covariates 
#' should be used to find DIF. Items are considered to be the response and are 
#' concatenated by \code{cbind()}. If the RHS of the formula is \code{~0}, simply the 
#' model specified in \code{model} is calulated.
#' @param data Data frame containing the ordinal item response data 
#' (as ordered factors) and all covariates.
#' @param DSF Should Differential Step Functioning (DSF) be considered? If \code{DSF = TRUE}, one parameter per step between two response categories is introduced. For binary items, DSF and DIF conincide. 
#' @param model Specify the underlying basic model. Currently, you can choose 
#' between the partial credit model and the rating scale model and
#' the respective generalized versions of both models called 'PCM', 'RSM', 
#' 'GPCM' and 'GRSM'. Generalized models allow for different discrimination 
#' parameters between items. 
#' @param control Control argument to specify further arguments for the algorithm
#' and numerical optimization, specified by \code{\link{ctrl_GPCMlasso}}.
#' @param cv Should cross-validation be performed? Cross-validation can be used as an alternative to BIC to select the optimal tuning parameter.
#' @param main.effects Should also main effects of the variables be included in the model? Default is \code{TRUE}. Here, positive parameter estimates correspond 
#' to an increase of the respective trait if the variable increases.
#' @return 
#' \item{coefficients}{Matrix containing all parameters for the GPCMlasso model, one row
#' per tuning parameter lambda. Due to the penalty the parameters are scaled and, therefore,
#' are comparable with respect to their size.} 
#' \item{logLik}{Vector of log-likelihoods, one value per tuning parameter lambda.}
#' \item{call}{The function call of \code{GPCMlasso}}
#' \item{cv_error}{Vector of cv_errors, one per tuning parameter. Only relevant if \code{cv = TRUE}.}
#' \item{model}{Basic IRT model chosen by user.}
#' \item{data}{Data from call.}
#' \item{control}{Control list.}
#' \item{DSF}{DSF from call.}
#' \item{formula}{Formula from call.}
#' \item{item.names}{Item names.}
#' \item{Y}{Matrix containing item responses.}
#' \item{design_list}{List containing several helpful objects for internal use.}
#' \item{AIC}{Vector of AIC values, one per tuning parameter.}
#' \item{BIC}{Vector of BIC values, one per tuning parameter.}
#' \item{cAIC}{Vector of corrected AIC values, one per tuning parameter.}
#' \item{df}{Vector of degrees of freedom, one per tuning parameter.}
#' \item{coef.rescal}{Matrix containing all rescaled parameters for the GPCMlasso model, one row
#' per tuning parameter lambda. In contrast to \code{coefficients}, all parameters are rescaled back
#' to their original scales.}
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @seealso \code{\link{GPCMlasso-package}}, \code{\link{ctrl_GPCMlasso}}, \code{\link{print.GPCMlasso}}, 
#' \code{\link{plot.GPCMlasso}}, \code{\link{predict.GPCMlasso}}, \code{\link{trait.posterior}}
#' @keywords GPCMlasso
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
GPCMlasso <- function(formula, data, DSF = FALSE, 
                      model = c("PCM","RSM","GPCM","GRSM","RM","2PL"),
                      control = ctrl_GPCMlasso(), cv = FALSE, main.effects = TRUE){

  ### save old contrasts to set them back to default later
  # old.contrasts <- options()$contrasts
  # options(contrasts = control$contrasts)
  # on.exit(options(contrasts = old.contrasts))
  
  ## check if data is data.frame
  if(!is.data.frame(data))
    stop("data has to be a data.frame!")
  
  ## apply model response and get Y
  Y <- model.response(model.frame(formula, data = data))
  ## factorize, also for binary Y is now 1,2,...
  for(i in 1:ncol(Y)){
    Y[,i] <- as.factor(Y[,i])
  }
  ## get item names
  item.names <- colnames(Y)
  
  ## get model, define respective dummies
  model <- match.arg(model)
  ## RSM makes rsm paramterization
  ## GPCM uses different instead of equal discrim parameters
  RSM <- GPCM <- FALSE
  if(model %in% c("GRSM","RSM")){
    RSM <- TRUE
  }
  if(model %in% c("GRSM","GPCM","2PL")){
    GPCM <- TRUE
  }

  ## for binary, DSF or cross-validation with deviance does not make sense
  k <- apply(Y, 2, function(x){length(levels(as.factor(x)))})
  
  if(all(k==2) | model %in% c("RM","2PL")){
    DSF <- FALSE
    control$cv.crit <- "deviance"
  }
  
  ## check if for RSM == TRUE all numbers of categories are equal
  if(length(unique(k))>1 & RSM){
    stop("Models RSM and GRSM cannot be used for unequal numbers of response categories!")
  }
  

  ## get design matrices and all further parameters  
  design_list <- design_GPCMlasso(formula = formula, data = data, 
                                  Y = Y, RSM = RSM, GPCM = GPCM, 
                                  DSF = DSF, all.dummies = control$all.dummies,
                                  main.effects = main.effects,
                                  penalize.main.effects = control$penalize.main.effects,
                                  fuse.per.variable = control$fuse.per.variable)

  
  ## if no covariates are used, we don't need lambda or adaptive
  if(design_list$m==0){
    control$lambda <- 0
    control$adaptive <- FALSE
    cv <- FALSE
  }
  if(length(control$lambda)==1){
    cv <- FALSE
  }
  
  ## which loglik and which score function is used,
  loglik_fun <- loglikPCMlasso
  score_fun <- scorePCMlasso
  log_score_fun <- loglikscorePCMlasso2
  if(all(k==2)){
    loglik_fun <- loglikDIFlasso
    score_fun <- scoreDIFlasso
    log_score_fun <- loglikscoreDIFlasso2
  }
  
  
  ## get fitted parameters and logliks
  fit <- fit_GPCMlasso(model = model,loglik_fun = loglik_fun, score_fun = score_fun, log_score_fun,
                       design_list = design_list, 
                       control = control,  start = NULL, scale_fac = 1, main.effects = main.effects)
  
  ## update lambda if lambda was not specified in advance
  if(is.null(control$lambda)){
    control$lambda <- fit$lambda
  }
# browser()
  ## extract coefficients
  coefficients <-  fit$coefficients
  
  coef.rescal <- fit$coef.rescal
  
  ##extract vector of logliks and degrees of freedom
  logLik <- fit$logLik
  df <- fit$df
  
  BIC <- -2*logLik + log(design_list$n)*df
  AIC <- -2*logLik + 2*df
  cAIC <- -2*logLik + 2*df +2*df*(df-1)/(design_list$n-df-1)
  

  ## apply cross validation if wanted
  if(cv){
    cv_error <- fit_cv_GPCMlasso(model, design_list, 
                                  control, score_fun,
                                  loglik_fun, log_score_fun, Y, main.effects)
  }else{
    cv_error <- NULL
  }

  ## create return list
  ret.list <- list(coefficients = coefficients, logLik = logLik, 
                   cv_error = cv_error, call = match.call(), 
                   model = model, data = data, control = control,
                   DSF = DSF, formula = formula,
                   item.names = item.names, Y = Y, design_list = design_list,
                   AIC = AIC, BIC = BIC, cAIC = cAIC, df = df, 
                   coef.rescal = coef.rescal, main.effects = main.effects)
  
  class(ret.list) <- "GPCMlasso"
  
  return(ret.list)
}

find.lambda <- function(cur.lambda, l.lambda, alpha.start, log_score_fun,
                        Q,q,I,n,response,design,designX, px, 
                        GHweights, GHnodes, acoefs, scale_fac, lambda2, cvalue, 
                        cores, weight, n_sigma, null_thresh, gradtol, iterlim, 
                        steptol){
  
  
  cur.low <- 0
  cur.up <- lambda.ratio <- Inf
  cur.lambda <- 0.5*cur.lambda

  while(lambda.ratio>0.15){
    suppressWarnings(
    m.opt <- try(nlm(log_score_fun, alpha.start,
                     Q = Q, q = q, I = I, n = n, Y = response, X = design, Z = designX, px = px,
                     GHweights = GHweights, GHnodes = GHnodes,
                     acoefs = acoefs, lambda = cur.lambda, scale_fac = scale_fac,
                     lambda2 = lambda2, cvalue = cvalue, cores = cores, weight = weight,  n_sigma = n_sigma,
                     gradtol = gradtol, iterlim = iterlim, check.analyticals = FALSE, 
                     steptol = steptol)))
  if(inherits(m.opt, "try-error")){
  stop("Determination of maximal tuning parameter failed. Please pre-specify
          the vector of tuning parameters manually via ctrl_GPCMlasso!")
    }

  coefs.cur <- m.opt$estimate%*%acoefs
  coefs.cur[abs(coefs.cur) < null_thresh] <-0

  if(sum(abs(coefs.cur))==0){
    cur.up <- cur.lambda
  }else{
    cur.low <- cur.lambda
  }
  if(is.finite(cur.up)){
    cur.lambda <- (cur.up-cur.low)*0.5+cur.low
    lambda.ratio <- (cur.up-cur.low)/cur.up
  }else{
    cur.lambda <- cur.lambda*2
  }

  ##end of while loop
  }
  return(cur.lambda)
}

get_designX <- function(X, DSF, m, I, q, n){
  if(!DSF){
    designX <- matrix(0, ncol = m * I, nrow = n * sum(q))
    
    pos_u <- 1
    for (u in 1:n) {
      for(uu in 1:I){
        designX[pos_u:(pos_u+q[uu]-1), ((uu - 1) * m + 1):(uu * m)] <-  
          matrix(rep(X[u,], q[uu]), byrow = TRUE, ncol = m, nrow = q[uu])
        pos_u <- pos_u+q[uu]
      }
    }
  }else{
    designX <- matrix(0, ncol = m * sum(q), nrow = n * sum(q))
    
    pos_u <- 1
    for (u in 1:n) {
      pos_uu <- 1
      for(uu in 1:I){
        for(uuu in 1:q[uu]){
          designX[pos_u, pos_uu:(pos_uu+m-1)] <-  
            X[u,]
          pos_u <- pos_u+1
          pos_uu <- pos_uu+m
        }
      }
    }
  }
  return(designX)
}


get_acoefs_old <- function(RSM, DSF, m, I, q, n_sigma){

  if(!DSF){
    pen1 <- diag(m*I)
  }else{
    pen1 <- matrix(0,nrow=m*sum(q),ncol=m*sum(choose(q,2)))
    pos1 <-1

    pos_pos <- 1
    for(u in 1:I){
      n_comb <- choose(q[u],2)
      if(n_comb>0){
      combis <- combn(q[u],2)-1
      for(uuu in 1:m){
        for(uu in 1:n_comb){
          pen1[combis[1,uu]*m+pos_pos,pos1] <- 1
          pen1[combis[2,uu]*m+pos_pos,pos1] <- -1
          pos1 <- pos1+1
        }
        pos_pos <- pos_pos+1
      }
      pos_pos <- pos_pos+(q[u]-1)*m
    }
    }
    pen1 <- cbind(diag(m*sum(q)),pen1)
  }
    if(RSM){
      acoefs <- rbind(matrix(0,nrow=q[1]+I-1,ncol=ncol(pen1)),pen1,
                      matrix(0, ncol = ncol(pen1), nrow = n_sigma))
      
    }else{
      acoefs <- rbind(matrix(0,nrow=sum(q),ncol=ncol(pen1)),pen1,
                      matrix(0, ncol = ncol(pen1), nrow = n_sigma))
    }
  }



get_acoefs_old2 <- function(RSM, DSF, m, I, q, n_sigma, main.effects,
                       penalize.main.effects){

  if(!DSF){
    pen1 <- diag(m*I)
  }else{
    pen1 <- matrix(0,nrow=m*sum(q),ncol=m*sum(q-1))
    pos1 <-1
    
    pos_pos <- 1
    for(u in 1:I){
      n_comb <- q[u] - 1
      if(n_comb>0){
        combis <- rep(1:q[u], each = 2)
        combis <- matrix(combis[-c(1,length(combis))]-1, nrow = 2)
        for(uuu in 1:m){
          for(uu in 1:n_comb){
            pen1[combis[1,uu]*m+pos_pos,pos1] <- 1
            pen1[combis[2,uu]*m+pos_pos,pos1] <- -1
            pos1 <- pos1+1
          }
          pos_pos <- pos_pos+1
        }
        pos_pos <- pos_pos+(q[u]-1)*m
      }
    }
    pen1 <- cbind(diag(m*sum(q)),pen1)
  }
  
  if(main.effects){
    if(penalize.main.effects){
      pen2 <- cbind(diag(m), matrix(0, ncol = ncol(pen1), nrow = m))
      pen1 <- cbind(matrix(0, ncol = m, nrow = nrow(pen1)), pen1)
      pen1 <- rbind(pen2, pen1)
    }else{
      pen1 <- rbind(matrix(0, ncol = ncol(pen1), nrow = m), pen1)
    }
  }
  
  
  ### added for fusion of DIF effects
  ## note: acoefs has one row per parameter, and one column per penalty
  ## what is needed if fusion is used? just add fusion penalties, or do we delete 
  ## other penalties?
  # if(fuse.per.variable){
  #   
  # }
  
  if(RSM){
    acoefs <- rbind(matrix(0,nrow=q[1]+I-1,ncol=ncol(pen1)),pen1,
                    matrix(0, ncol = ncol(pen1), nrow = n_sigma))
    
  }else{
    acoefs <- rbind(matrix(0,nrow=sum(q),ncol=ncol(pen1)),pen1,
                    matrix(0, ncol = ncol(pen1), nrow = n_sigma))
  }
}

get_acoefs <- function(RSM, DSF, m, I, q, n_sigma, main.effects,
                       penalize.main.effects, fuse.per.variable){
  
  
  if(!DSF){
    pen1 <- diag(m*I)
    
    
    ### added for fusion of DIF effects
    if(fuse.per.variable){
      all.pairs <- combn(I,2)
      pen3 <- matrix(0, nrow = m*I, ncol = I*(I-1)/2)
      for(i in 1:ncol(all.pairs)){
        pen3[((all.pairs[1,i]-1)*m)+1,i] <- 1
        pen3[((all.pairs[2,i]-1)*m)+1,i] <- -1
      }
      if(m>1){
        pen4 <- pen3
        for(j in 2:m){
          pen4 <- rbind(rep(0,ncol(pen4)), pen4[-nrow(pen4),])
          pen3 <- cbind(pen3, pen4)
        }
      }
      
      pen1 <- cbind(pen1, pen3)
    }
    
  }else{
    if(fuse.per.variable){
      stop("If Differential Step Functioning is detected (argument DSF = TRUE),
                the option fuse.per.variable cannot be set TRUE.")
    }
    
    pen1 <- matrix(0,nrow=m*sum(q),ncol=m*sum(q-1))
    pos1 <-1
    
    pos_pos <- 1
    for(u in 1:I){
      n_comb <- q[u] - 1
      if(n_comb>0){
        combis <- rep(1:q[u], each = 2)
        combis <- matrix(combis[-c(1,length(combis))]-1, nrow = 2)
        for(uuu in 1:m){
          for(uu in 1:n_comb){
            pen1[combis[1,uu]*m+pos_pos,pos1] <- 1
            pen1[combis[2,uu]*m+pos_pos,pos1] <- -1
            pos1 <- pos1+1
          }
          pos_pos <- pos_pos+1
        }
        pos_pos <- pos_pos+(q[u]-1)*m
      }
    }
    pen1 <- cbind(diag(m*sum(q)),pen1)
  }
  
  if(main.effects){
    if(penalize.main.effects){
      pen2 <- cbind(diag(m), matrix(0, ncol = ncol(pen1), nrow = m))
      pen1 <- cbind(matrix(0, ncol = m, nrow = nrow(pen1)), pen1)
      pen1 <- rbind(pen2, pen1)
    }else{
      pen1 <- rbind(matrix(0, ncol = ncol(pen1), nrow = m), pen1)
    }
  }
  
  
  if(RSM){
    acoefs <- rbind(matrix(0,nrow=q[1]+I-1,ncol=ncol(pen1)),pen1,
                    matrix(0, ncol = ncol(pen1), nrow = n_sigma))
    
  }else{
    acoefs <- rbind(matrix(0,nrow=sum(q),ncol=ncol(pen1)),pen1,
                    matrix(0, ncol = ncol(pen1), nrow = n_sigma))
  }
}

## from a vector of the linear predictor, get the respective
## probabilities for a adjacent categories model
responseFun <- function(eta) {
  q <- length(eta)
  eta[eta > 10] <- 10
  eta[eta < -10] <- -10
  eta.help <- matrix(rep(c(0, eta), each = q + 1), ncol = q + 
    1)
  eta.help[upper.tri(eta.help)] <- 0
  pi <- cumprod(c(1, exp(eta[-q])))/sum(apply(exp(eta.help), 
    1, prod))
  pi <- (pi - 0.5) * 0.99999 + 0.5
  pi
}


## Get likelihood contribution of one person, given an ability
## (theta), a vector conaining the linear predictor and the
## response vector.  For cross-validation, the test values are
## already eliminated.
likeli_i <- function(theta, lin.pred, sigma, yp, q) {
  
  lin.pred <- lin.pred + rep(sigma,q)*theta
  lin.pred <- split(lin.pred, rep(1:length(yp),q))
  
  likeli.i <- 1
  for (i in 1:length(lin.pred)) {
    mu_i <- responseFun(lin.pred[[i]])
    mu_i <- c(mu_i, 1 - sum(mu_i))
    likeli.i <- mu_i[yp[i]] * likeli.i
  }
  likeli.i
}

## Create vector of linear predictors for all observations of
## the model.  Linear predictor ALSO contains discrimination
## parameters.
lin_pred <- function(model, coef_short, sigma) {
  
  with(model$design_list, {
    ## create design matrix out ouf item-parameter part and
    ## covariate part

    des_total <- matrix(rep(t(design),n),byrow=TRUE,ncol=ncol(design))
    if (ncol(designX) > 0) {
      des_total <- cbind(des_total, designX)
    }
    
    ## multiply design matrix by discrimination parameters, so
    ## that discrimination parameters are included in linear
    ## predictor.
      des_total <- rep(rep(sigma,q), n) * des_total

    lin.pred <- des_total %*% coef_short
    
    return(lin.pred)
  })
  
}


## vectorized version (with respect to x, which is ability) of
## the function to be integrated for person abilities.
## Vectorization is favorable for integrate.
estim_i <- function(x, yp, lin.pred, sigma, q) {

  dens.norm <- dnorm(x)
  lik.i <- sapply(x, likeli_i, lin.pred, sigma, yp, q)
  ret <- dens.norm * lik.i * x
  return(ret)
}

## equivalent to estim_i except that x is not multiplied,
## necessary for scaling
factor_i <- function(x, yp, lin.pred, sigma, q) {
  
  dens.norm <- dnorm(x)
  lik.i <- sapply(x, likeli_i, lin.pred, sigma, yp, q)
  ret <- dens.norm * lik.i
  
  return(ret)
}

## Function to estimate person ability parameter.  
## integrate and pcubature are almost equally fast
person_i <- function(person, person.index, all.lin.preds, sigma, 
  Y, tol, limit, q) {

  yp <- Y[person, ]
  sigma <- sigma[!is.na(yp)]
  q <- q[!is.na(yp)]
  yp <- yp[!is.na(yp)]
  
  lin.pred <- all.lin.preds[person.index == person]
  
  est_i <- pcubature(estim_i, lowerLimit = -limit, upperLimit = limit, 
                     yp = yp, lin.pred = lin.pred, sigma = sigma, q = q, 
                     tol = tol)
  
  est_i <- est_i$integral/pcubature(factor_i, lowerLimit = -limit, 
                                 upperLimit = limit, yp = yp, lin.pred = lin.pred, sigma = sigma, 
                                 q = q, tol = tol)$integral
  
  
  est_i
}

#' Calculate Posterior Estimates for Trait Parameters
#' 
#' Calculates posterior estimates for trait/person parameters using the assumption 
#' of Gaussian distributed parameters.
#' 
#' 
#' @param model Object of class \code{GPCMlasso}. 
#' @param coefs Vector of coefficients to be used for prediction. If \code{coefs = NULL}, 
#' the parameters from  the BIC-optimal model will be used. 
#' If cross-validation was performed, automatically the parameters from the optimal
#' model according to cross-validation are used. 
#' @param cores Number of cores to be used in parallelized computation.
#' @param tol The maximum tolerance for numerical integration, 
#' for more details see \code{\link[cubature]{pcubature}}.
#' @return Vector containing all estimates of trait/person parameters.
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @seealso \code{\link{GPCMlasso}} \code{\link{GPCMlasso-package}}
#' @keywords GPCMlasso
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
trait.posterior <- function(model, coefs = NULL, cores = 25, tol = 1e-4) {
  
  n <- model$design_list$n
  I <- model$design_list$I
  q <- model$design_list$q
  n_sigma <- model$design_list$n_sigma
  
  ## automatically get coefs if not provided
  if(is.null(coefs)){
    if(!is.null(model$cv_error)){
      cat("No coefs are provided, automatically cv-optimal model is chosen","\n")
      coefs <- model$coefficients[which.min(model$cv_error),]
    }else{
      cat("No coefs are provided, automatically BIC-optimal model is chosen","\n")
      coefs <- model$coefficients[which.min(model$BIC),]#
    }
  }
  

  ## separate sigma and other coefficients
  coef_short <- head(coefs, length(coefs)-n_sigma)
  sigma <- tail(coefs, n_sigma)
  if(n_sigma==1){
    sigma <- rep(sigma, I)
  }
    
  ## create person index
  person.index <- rep(1:n, each = sum(q))
  
  Y <- matrix(as.numeric(as.matrix(model$Y)), ncol = ncol(model$Y))
  
  ## create linear predictor without theta
  all.lin.preds <- lin_pred(model, coef_short, sigma)  

  estimates <- person.fit(n, I, q, Y, sigma, person.index, 
    all.lin.preds, cores, tol)
  
  names(estimates) <- rownames(model$data)
  
  estimates
}


person.fit <- function(n, I, q, Y, sigma, person.index, 
  all.lin.preds, cores = 1, tol = 1e-3) {
  
  
  limit <- qnorm(0.999)
  
  if (cores > 1) {
    cl <- makeCluster(cores, outfile = "")
    
    clusterExport(cl, varlist = c("Y", "factor_i", "person.index", 
      "all.lin.preds", "sigma", "q", "estim_i", "likeli_i", 
       "tol", "limit"), 
      envir = sys.frame(sys.nframe()))
    
    estimates <- parSapply(cl, seq(n), person_i, person.index = person.index, 
      all.lin.preds = all.lin.preds, sigma = sigma, Y = Y, 
      tol = tol, limit = limit, q = q)
    stopCluster(cl)
  } else {
    estimates <- sapply(seq(n), person_i, person.index = person.index, 
      all.lin.preds = all.lin.preds, sigma = sigma, Y = Y, 
      tol = tol, limit = limit, q = q)
  }
  

  estimates
}

## create responses for acat from ordinal values
createResponse <- function(Y){
  Y <- as.factor(Y)
  model.matrix(~0+Y)[,-length(levels(Y))]
}

design_GPCMlasso <- function(formula = formula, Y=Y, data = data, RSM = RSM, 
                             GPCM = GPCM, DSF = DSF, all.dummies = TRUE, 
                             main.effects = main.effects, penalize.main.effects,
                             fuse.per.variable){

  ## extract covariates
  if(all.dummies){
    term.labels <- attr(terms(formula), "term.labels")
    X <- matrix(rep(1,nrow(data)))
    for(ij in 1:length(term.labels)){
      x.now <- data[,term.labels[ij], drop = FALSE]
      if(is.factor(x.now[,1])){
        if(nlevels(x.now[,1])==2){
          X <- cbind(X,model.matrix(~.,data=x.now)[,-1,drop=FALSE])
        }else{
          X <- cbind(X,model.matrix(~0+.,data=x.now))
        }
      }else{
        X <- cbind(X,model.matrix(~.,data=x.now)[,-1,drop=FALSE])
      }
    }
     X <- X[,-1,drop = FALSE]
  }else{
    X <- model.matrix(formula, data = data)
    if(ncol(X)>=1){
      if(colnames(X)[1]=="(Intercept)"){
        X <- X[,-1,drop = FALSE]
      }
    }
  }


  x.names <- colnames(X)
  
  ## factorize response vector 
 
  
  ## initialize basic parameters
  k <- apply(Y, 2, function(x){length(levels(as.factor(x)))})
  q <- k-1
  n <- nrow(Y)
  I <- ncol(Y)
  m <- ncol(X)
  n_sigma <- 1
  if(GPCM){
    n_sigma <- I
  }

  ## get final response vector
  if(all(q==1)){
    response <- as.numeric(as.factor(c(t(Y))))-1
  }else{
    response <- lapply(as.data.frame(Y),createResponse)
    response <- c(t(do.call("cbind",response)))
  }
  
  ## total number of parameters to be optimized
  px <- sum(q)+n_sigma
  if(RSM){
    px <- q[1]+I+n_sigma-1
  }
  
  ## scale X and save standard deviations for re-scaling
  # X <- scale(X, center = FALSE)
  X <- scale(X, center = FALSE, scale = apply(X, 2, sd, na.rm = TRUE))
  sd.vec <- attributes(X)$scale
  
  sd.vec <- create.sd.vec(sd.vec, DSF, px, n_sigma, I, q, main.effects) 
  
  ## create design matrix for basic item parameters
  if(!RSM){
    design <- -diag(px-n_sigma)
  }else{
    design <- -cbind(matrix(rep(diag(I),each=q[1]),ncol=I),
               matrix(rep(diag(q[1]),I),ncol=q[1],byrow = TRUE))[,-(I+1)]
  }
  
  ## create design matrix for covariate part
  if(m>=1){
    designX <- -get_designX(X, DSF, m, I, q, n)
    
    ## create penalization matrix
    acoefs <- get_acoefs(RSM, DSF, m, I, q, n_sigma, main.effects,
                         penalize.main.effects, fuse.per.variable)

        ## update number of parameters to be optimized
    px <- px + ncol(designX)
  }else{
    ## dummy matrix in case no covariates are specified
    designX <- matrix(0,0,0)
    acoefs <- matrix(0,nrow=px,ncol=1)
  }
  
  if(main.effects){
    design.main <- matrix(rep(X, each = sum(q)),ncol = ncol(X))
    designX <- cbind(design.main, designX)
    #acoefs <- rbind(matrix(0, ncol = ncol(acoefs), nrow = ncol(design.main)), acoefs)
    px <- px + ncol(X)
  }
  

  ret.list <- list(q = q, I = I, m = m, px = px, n = n, response = response,
                   design = design, designX = designX, sd.vec = sd.vec,
                   acoefs = acoefs, n_sigma = n_sigma, x.names = x.names,
                   RSM = RSM, GPCM = GPCM, Y = Y, DSF = DSF)
  
  return(ret.list)
}

create.sd.vec <- function(sd.vec, DSF, px, n_sigma, I, q, main.effects){

  if(DSF){
    new_vec <- rep(sd.vec, sum(q))
  }else{
    new_vec <- rep(sd.vec, I)
  }
  if(main.effects){
    new_vec <- c(sd.vec, new_vec)
  }
  new_vec <- c(rep(1,px-n_sigma),new_vec,rep(1,n_sigma))
  new_vec
}

#' Control function for GPCMlasso
#' 
#' Control parameters for penalty terms and for tuning the fitting algorithm.
#' 
#' @param log.lambda Should the grid of tuning parameters be created on a log scale?
#' @param lambda Optional argument to specify a vector of tuning parameters. If \code{lambda = NULL}, a vector of length \code{l.lambda} is created automatically. 
#' @param l.lambda Specifies the length of the grid of tuning parameters.
#' @param lambda.min Minimal value used if the grid of tuning parameters is created automatically.
#' @param penalize.main.effects Should also main covariate effects be penalized? Default is \code{FALSE}. 
#' @param fuse.per.variable Should fusion be applied per variable? This option 
#' creates clusters of items with equal effects for a variable. 
#' @param adaptive Should adaptive lasso be used? Default is \code{TRUE}.
#' @param weight.penalties Should penalties be weighted according to the number of penalty term and the number of parameters
#' corresponding to one pair between item and covariate. Only relevant if both \code{DSF = TRUE} and the number of 
#' response categories differs across items (because only then these values can differ).  
#' @param ada.lambda Size of tuning parameter for Ridge-regularized estimation of parameters used for adaptive weights.
#' @param ada.power By default, 1st power of absolute values of Ridge-regularized estimates are used. Could be changed to squared values by \code{ada-power = 2}.
#' @param Q Number of nodes to be used in Gauss-Hermite quadrature.
#' @param lambda2 Tuning parameter for ridge penalty on all coefficients except 
#' sigma/slope parameters.
#' Should be small, only used to stabilize results.
#' @param cvalue Internal parameter for the quadratic approximation of the L1
#' penalty. Should be sufficiently small. 
#' @param trace Should the trace of the progress (current tuning parameter) be printed?
#' @param folds Number of folds for cross-validation. Only relevant if \code{cv = TRUE} in \code{\link{GPCMlasso}}.
#' @param cores Number of cores to be used parallel when fitting the model.
#' @param null_thresh Threshold which is used to distinguih between values equal and unequal to zero.
#' @param gradtol Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.
#' @param steptol Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.
#' @param iterlim Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.
#' @param precision Number of decimal places used to round coefficient estimates. 
#' @param all.dummies Should (in case of factors with more than 2 categories) the dummy variables for all categories be included in the design matrix? If \code{all.dummies = TRUE}, the dependence on the reference category is eliminated for multi-categorical covariates.
#' @param ctrl.gpcm List of control arguments for initial \code{\link[ltm]{gpcm}} fit, which is needed to get good starting values. Does not apply to RSM or GRSM.  
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
ctrl_GPCMlasso <- function(log.lambda = TRUE, lambda = NULL, l.lambda = 50,
                           lambda.min = 0.1, penalize.main.effects = FALSE,
                           fuse.per.variable = FALSE, 
                           adaptive = TRUE, weight.penalties = TRUE, ada.lambda = 1e-4, ada.power = 1,
                           Q = 15, lambda2 = 1e-4, cvalue = 1e-5, 
                           trace = TRUE, folds = 10, cores = 25, 
                           null_thresh = 0.01, gradtol = 1e-6,
                           steptol = 1e-6, iterlim = 500, 
                            precision = 3,
                           all.dummies = FALSE, ctrl.gpcm = list()){
  
  RET <- list(log.lambda = log.lambda, lambda = lambda, l.lambda = l.lambda, 
              cores = cores, adaptive = adaptive, weight.penalties = weight.penalties, ada.lambda = ada.lambda, Q = Q, 
              lambda2 = lambda2, cvalue = cvalue, trace = trace, ada.power = ada.power,
              folds = folds, null_thresh = null_thresh,
              gradtol = gradtol, steptol = steptol, iterlim = iterlim, 
              lambda.min = lambda.min, precision = precision, 
              all.dummies = all.dummies, penalize.main.effects = penalize.main.effects,
              fuse.per.variable = fuse.per.variable, ctrl.gpcm = ctrl.gpcm)
  RET
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

loglikPCMlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_loglikPCMlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

scorePCMlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_scorePCMlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

loglikDIFlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_loglikDIFlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

scoreDIFlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_scoreDIFlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

loglikscorePCMlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_loglikscorePCMlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

loglikscoreDIFlasso <- function(alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac) {
    .Call(`_GPCMlasso_loglikscoreDIFlasso`, alpha, Y, X, Z, Q, q, n, I, px, GHweights, GHnodes, acoefs, lambda, lambda2, cvalue, cores, weight, n_sigma, scale_fac)
}

#' Find DIF in Generalized Partial Credit Models
#' 
#' Performs GPCMlasso, a method to identify DIF in Generalized Partial Credit Models. 
#' A joint parametric model is set up based on an IRT model chosen by the user. Several variables can be considered simultaneously.
#' For each pair between variable and item, a parametric DIF effect is introduced which
#' indicates DIF if the respective parameter is selected (estimated to be unequal zero). 
#' Parameter selection is done using a lasso-type penalization term.
#' 
"_PACKAGE"
#' @name GPCMlasso-package
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @seealso \code{\link{GPCMlasso}}
#' @keywords package Partial Credit DIF GPCMlasso GPCM DSF
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
NULL


#' Tenseness data from the Freiburg Complaint Checklist
#' 
#' Data from the Freiburg Complaint Checklist. 
#' The data contain all 8 items corresponding to the scale \emph{Tenseness} for 2042 participants of the 
#' standardization sample of the Freiburg Complaint Checklist. 
#' 
#' @name tenseness
#' @docType data
#' @format A data frame containing data from the Freiburg Complaint Checklist with 1847 observations. 
#' All items refer to the scale \emph{Tenseness} and are measured on a 5-point Likert scale where low numbers 
#' correspond to low frequencies or low intensitites of the respective complaint and vice versa. 
#' \describe{ 
#' \item{Clammy_hands}{Do you have clammy hands?}
#' \item{Sweat_attacks}{Do you have sudden attacks of sweating?}
#' \item{Clumsiness}{Do you notice that you behave clumsy?}
#' \item{Wavering_hands}{Are your hands wavering frequently, e.g. when lightning a cigarette or when holding a cup?}
#' \item{Restless_hands}{Do you notice that your hands are restless?}
#' \item{Restless_feet}{Do you notice that your feet are restless?}
#' \item{Twitching_eyes}{Do you notice unvoluntary twitching of your eyes?}
#' \item{Twitching_mouth}{Do you notice unvoluntary twitching of your mouth?}
#' \item{Gender}{Gender of the person}
#' \item{Household}{Does the person live alone in a household or together with somebody?}
#' \item{Income}{Income, categorized to levels from 1 (low income) to 11(high income). For simplicity,
#' due to the high number of categories income can be treated as a metric variable.}
#' \item{WestEast}{Is the person from East Germany (former GDR)?}
#' \item{Abitur}{Does the person have Abitur (A-levels)?}
#' \item{Age}{Age of the person}
#'  }
#' @source 
#' ZPID (2013). PsychData of the Leibniz Institute for Psychology Information ZPID. Trier: Center for Research Data in Psychology.
#' 
#' Fahrenberg, J. (2010). Freiburg Complaint Checklist [Freiburger Beschwerdenliste (FBL)]. Goettingen, Hogrefe.
#' @keywords datasets
#' @examples
#' 
#' data(tenseness)
#' 
NULL

#' Subset of tenseness data from the Freiburg Complaint Checklist
#' 
#' Data from the Freiburg Complaint Checklist. 
#' The data contain 5 items (out of 8) corresponding to the scale \emph{Tenseness} for a subset of 200 participants of the 
#' standardization sample of the Freiburg Complaint Checklist. 
#' 
#' @name tenseness_small
#' @docType data
#' @format A data frame containing data from the Freiburg Complaint Checklist a subset of 200 observations. 
#' The complete data set with 1847 observations can be found in \code{\link{tenseness}}.
#' All items refer to the scale \emph{Tenseness} and are measured on a 5-point Likert scale where low numbers 
#' correspond to low frequencies or low intensitites of the respective complaint and vice versa. 
#' \describe{ 
#' \item{Clammy_hands}{Do you have clammy hands?}
#' \item{Sweat_attacks}{Do you have sudden attacks of sweating?}
#' \item{Clumsiness}{Do you notice that you behave clumsy?}
#' \item{Wavering_hands}{Are your hands wavering frequently, e.g. when lightning a cigarette or when holding a cup?}
#' \item{Restless_hands}{Do you notice that your hands are restless?}
#' \item{Gender}{Gender of the person}
#' \item{Age}{Age of the person}
#'  }
#' @seealso \code{\link{GPCMlasso}}, \code{\link{ctrl_GPCMlasso}}, \code{\link{trait.posterior}}
#' @source 
#' ZPID (2013). PsychData of the Leibniz Institute for Psychology Information ZPID. Trier: Center for Research Data in Psychology.
#' 
#' Fahrenberg, J. (2010). Freiburg Complaint Checklist [Freiburger Beschwerdenliste (FBL)]. Goettingen, Hogrefe.
#' @keywords GPCMlasso
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
NULL

#' Predict function for GPCMlasso
#' 
#' Predict function for a \code{GPCMlasso} object. 
#' Predictions can be linear predictors or probabilities separately for each person and each item.
#' 
#' Results are lists of vectors with length equal to the number 
#' of response categories $k_i$ in case of
#' probabilities (\code{type="response"}) or 
#' $k_i-1$ in case of linear predictors (\code{type="link"}).
#' 
#' @param object \code{GPCMlasso} object
#' @param coefs Optional vector of coefficients, can be filled with a specific 
#' row from \code{object$coefficients}. If not specified, \code{coefs} are specififed to be
#' the BIC-optimal coefficients or, if cross-validation was performed, the optimal 
#' coefficients according to cross-validation.
#' @param newdata List possibly containing slots Y, X, Z1 and Z2 to use new data for prediction.
#' @param type Type "link" gives vectors of linear predictors for separate categories 
#' (of length $k_i-1$) and type "response" gives the respective probabilities (of length $k_i$).  
#' @param ... Further predict arguments.
#' @author Gunther Schauberger\cr \email{gunther@@stat.uni-muenchen.de}
#' @seealso \code{\link{GPCMlasso}}
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
predict.GPCMlasso <- function(object, coefs = NULL,
                              newdata = NULL,
                              type = c("link","response"),...){
  type <- match.arg(type)
  
  m <- object$design_list$m
  n <- object$design_list$n
  n_sigma <- object$design_list$n_sigma
  I <- object$design_list$I
  q <- object$design_list$q
  design <- object$design_list$design
  
  ## automatically get coefs if not provided
  if(is.null(coefs)){
    if(!is.null(object$cv_error)){
      cat("No coefs are provided, automatically cv-optimal model is chosen","\n")
      coefs <- object$coefficients[which.min(object$cv_error),]
    }else{
      cat("No coefs are provided, automatically BIC-optimal model is chosen","\n")
      coefs <- object$coefficients[which.min(object$BIC),]#
    }
  }
 
    

    ## extract covariates and response
  if(!is.null(newdata)){

  if(object$control$all.dummies){
    term.labels <- attr(terms(object$formula), "term.labels")
    X <- matrix(rep(1,nrow(newdata)))
    for(ij in 1:length(term.labels)){
      x.now <- newdata[,term.labels[ij], drop = FALSE]
      if(is.factor(x.now[,1])){
        if(nlevels(x.now[,1])==2){
          X <- cbind(X,model.matrix(~.,data=x.now)[,-1,drop=FALSE])
        }else{
          X <- cbind(X,model.matrix(~0+.,data=x.now))
        }
      }else{
        X <- cbind(X,model.matrix(~.,data=x.now)[,-1,drop=FALSE])
      }
    }
    X <- X[,-1,drop = FALSE]
  }else{
    X <- model.matrix(object$formula, data = newdata)
    if(ncol(X)>=1){
      if(colnames(X)[1]=="(Intercept)"){
        X <- X[,-1,drop = FALSE]
      }
    }
  }
    designX <- -get_designX(X, object$DSF, m, I, q, nrow(X))
  }else{
    designX <- object$design_list$designX
  }

     # browser()
    RSM <- GPCM <- FALSE
    if(object$model %in% c("GRSM","RSM")){
      RSM <- TRUE
    }
    if(object$model %in% c("GRSM","GPCM")){
      GPCM <- TRUE
    }
    
    des_total <- matrix(rep(t(design),n),byrow=TRUE,ncol=ncol(design))
    if(ncol(designX)>0){
      des_total <- cbind(des_total, designX)
    }
    
    coef_short <- head(coefs, length(coefs)-n_sigma)
    lin.pred <- des_total%*%coef_short
    
    theta <- trait.posterior(object, coefs, cores = object$control$cores)

    ### old version not taking into account beta_i
    return_list <- rep(theta,each=sum(q))+lin.pred
    
    ## new version with beta_i
    beta_i <- coefs[(length(coefs)-n_sigma+1):length(coefs)]
    if(!GPCM){
      beta_i <- rep(beta_i,I)
    }
    return_list <- rep(rep(beta_i,q),n) * (rep(theta,each=sum(q))+lin.pred)
    return_list <- split(return_list, rep(rep(1:I,q),n)+(rep(1:n,each=sum(q))-1)*I)
    
    if(type=="response"){
      for(i in 1:length(return_list)){
        pi.i <- responseFun(return_list[[i]])
        return_list[[i]] <- c(pi.i,1-sum(pi.i))
      }
    }
      
      names(return_list) <- paste(rep(paste0("Person_",1:n),each=I),rep(object$item.names,n),sep="_")
    return_list
    
  
}

#' Print function for GPCMlasso
#' 
#' Print function for a \code{GPCMlasso} object. Prints parameters estimates for all model
#' components for the optimal model chosen by a specific criterion (by default BIC). 
#' 
#' @param x \code{GPCMlasso} object
#' @param select Specifies which criterion to use for the optimal model, we recommend the 
#' default value "BIC". If cross-validation was performed, automatically the optimal
#' model according to cross-validation is used. Only the parameter estimates from
#' the chosen optimal model are printed.
#' @param ... Further print arguments.
#' @author Gunther Schauberger\cr \email{gunther.schauberger@@tum.de}
#' @references Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
#' Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
#' @seealso \code{\link{GPCMlasso}}
#' @examples
#' data(tenseness_small)
#' 
#' ## formula for simple model without covariates
#' form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))
#' 
#' ######
#' ## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
#' rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
#' control= ctrl_GPCMlasso(cores=2))
#' rsm.0
#' 
#' \dontrun{
#' ## formula for model with covariates (and DIF detection)
#' form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))
#' 
#' ######
#' ## fit GPCM model with 10 different tuning parameters
#' gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
#'                   control = ctrl_GPCMlasso(l.lambda = 10))
#' gpcm
#' plot(gpcm)
#' pred.gpcm <- predict(gpcm)
#' trait.gpcm <- trait.posterior(gpcm)
#' 
#' ######
#' ## fit RSM, detect differential step functioning (DSF)
#' rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
#'                      control = ctrl_GPCMlasso(l.lambda = 10))
#' rsm.DSF
#' plot(rsm.DSF)
#' 
#' ## create binary data set
#' tenseness_small_binary <- tenseness_small
#' tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2
#' 
#' ######
#' ## fit and cross-validate Rasch model
#' set.seed(1860)
#' rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
#'                    control = ctrl_GPCMlasso(l.lambda = 10))
#' rm.cv
#' plot(rm.cv)
#' }
print.GPCMlasso <- function(x, select = c("BIC", "AIC", "cAIC", "cv"), ...){
  with(x$design_list,{

    select <- match.arg(select, c("BIC", "AIC", "cAIC", "cv"))

    if(select=="BIC"){
      criterion <- x$BIC
    }
    if(select=="AIC"){
      criterion <- x$AIC
    }
    if(select=="cAIC"){
      criterion <- x$cAIC
    }
    if(select=="cv" | !is.null(x$cv_error)){
      criterion <- x$cv_error
    }
    
      coefs <- x$coefficients[which.min(criterion),]
      loglik <- x$logLik[which.min(criterion)]
# browser()

  if(RSM){
    delta <- coefs[1:I]
    names(delta) <- x$item.names
    alpha <- c(0,coefs[(I+1):(I+q[1]-1)])
    names(alpha) <- paste0("Catgr. ",1:(q[1]))
    start.gamma <- start.sigma <- I+q[1]
  }else{
    if(x$model %in% c("RM","2PL")){
      delta <- coefs[1:I]
      names(delta) <- x$item.names
      alpha <-NA
      start.gamma <- start.sigma <- I+1
    }else{
      delta <- coefs[1:sum(q)]
      if(sum(diff(q))==0){
        delta <- matrix(delta, byrow = TRUE, ncol = q[1])
        rownames(delta) <- x$item.names
        colnames(delta) <- paste0("Catgr. ",1:(q[1]))
      }else{
        delta <- split(delta,rep(1:I,q))
        names(delta) <- x$item.names
        for(ii in 1:I){
          names(delta[[ii]]) <- paste0("Catgr. ",1:(q[ii]))
        }
      }
      alpha <- NA
      start.gamma <- start.sigma <- sum(q)+1
    }
  }
  
  if(x$main.effects & m > 0){
    mains <- coefs[start.gamma:(start.gamma+m-1)]
    names(mains) <- x.names
    start.gamma <- start.sigma <- start.gamma + m
  }else{
    mains <- NA
  }    
      
  n.dif.par <- I*m
  if(x$DSF){
    n.dif.par <- sum(q)*m
  }

  if(m>0){
    if(x$DSF){
      if(sum(diff(q))==0){
        gamma <- matrix(coefs[start.gamma:(start.gamma + n.dif.par - 1)], byrow = TRUE, nrow = I)
        rownames(gamma) <- x$item.names
        colnames(gamma) <- paste(rep(x.names, q[1]), rep(1:q[1], each = length(x.names)), sep = ".")
        index.gamma <- c(t(matrix(1:(m*q[1]), nrow = m)))
        gamma <- gamma[, index.gamma]
      }else{
        gamma <- list()
        coefs.pen <- coefs[start.gamma:(start.gamma+n.dif.par-1)]
        start.coefs <- 1
        for(i in 1:I){
          coefs.i <- coefs.pen[start.coefs:(start.coefs-1+q[i]*m)]
          gamma.index <- matrix(1:(q[i]*m),nrow=m)
          gamma.i <- c()
          for(ii in 1:m){
            coefs.ii <- coefs.i[gamma.index[ii,]]
            names(coefs.ii) <- paste0(x.names[ii],".",1:(q[i]))
            gamma.i <- c(gamma.i,coefs.ii)
          }
          gamma[[i]] <- gamma.i
          start.coefs <- q[i]*m+start.coefs
        }
      names(gamma) <- x$item.names
      }
    }else{
      gamma <- matrix(coefs[start.gamma:(start.gamma+n.dif.par-1)],byrow=TRUE,nrow=I)
      rownames(gamma) <- x$item.names
      colnames(gamma) <- x.names
    }
    start.sigma <- start.gamma+n.dif.par
  }else{
    gamma <- NA
  }
  
  if(GPCM){
    sigma <- coefs[start.sigma:(start.sigma+I-1)]
    names(sigma) <- x$item.names
  }else{
    sigma <- coefs[start.sigma]
  }
  
  cat("Call: ")
  print(x$call,...)
  cat("\n")
  
  cat("Item parameters:","\n")
  print(delta, ...)
  
  
  
  suppressWarnings(if(!is.na(alpha[1])){
    cat("\n")
    cat("Category parameters:","\n")
    print(alpha, ...)
  cat("\n")  })
  
  suppressWarnings(if(!is.na(mains[1])){
    cat("\n")
    cat("Main covariate effects:","\n")
    print(mains, ...)
  cat("\n")  })
  
  
  suppressWarnings(if(!is.na(gamma[1])){
    cat("\n")
    cat("DIF parameters:","\n")
    print(gamma, ...)
  })
  cat("\n")
  
  cat("Discrimination parameter(s):","\n")
  cat(sigma,"\n")
  
  cat("\n")
  
  cat("Log-likelihood:","\n")
  cat(loglik)
  cat("\n")
  
  invisible(list(x = x, gamma = gamma))
  })

}

responseFun2 <- function(eta) {
  q <- length(eta)

  eta.help <- matrix(rep(c(0, eta), each = q + 1), ncol = q + 
                       1)
  eta.help[upper.tri(eta.help)] <- 0
  pi <- cumprod(c(1, exp(eta[-q])))/sum(apply(exp(eta.help), 
                                              1, prod))

  pi
}


sim_fun <- function(model, m, I, k, n, gamma, seed = NULL){

  if(!is.null(seed)){
    set.seed(seed)
  }
  
  RSM <- GPCM <- FALSE
  if(model %in% c("GRSM","RSM")){
    RSM <- TRUE
  }
  if(model %in% c("GRSM","GPCM","2PL")){
    GPCM <- TRUE
  }
  
  
  q <- k-1

  X <- c()
  for(i in 1:m){
    if(i%%2 == 1){
      X <- cbind(X, rnorm(n))
    }else{
      X <- cbind(X, rbinom(n,1,0.5))
    }
  }
  X <- scale(X)
  
  if(!RSM){
    delta <- deltaX <- matrix(round(rnorm(q*I,sd=0.5),2),nrow=I)
    # if(model == "GPCM"){
    #   delta <- deltaX <- delta-0.5
    # }
    alpha <- NA
  }else{
    delta <- round(rnorm(I,sd=0.5),2)
    alpha <- c(0,round(rnorm(q-1,sd=0.5),2))
    deltaX <- t(t(matrix(rep(delta,q),nrow=I))+alpha)
  }
  
  
  if(!GPCM){
    sigma <- rep(1,I)
  }else{
    sigma <- seq(0.7,1,length=I)
  }
  
  theta <- rnorm(n)
  
  
  lin_pred<- c()
  probs <- c()
  y <- c()
  
  for(i in 1:n){
    for(ii in 1:I){
      eta <- sigma[ii] * (theta[i] - deltaX[ii,] -sum(gamma[ii,]*X[i,]))
      
      lin_pred <- rbind(lin_pred, eta)
      pi <- responseFun2(eta)
      if(q==1){
        pi <- exp(eta)/(1+exp(eta))
      }
      probs <- rbind(probs,pi)
      pi <- c(pi,1-sum(pi))

      # if(any(pi<0)){browser()}
      y.sample <- which(rmultinom(1,1,pi)==1) 
      y <- c(y,y.sample)
    }
  }
  
  Y <- matrix(y,byrow=TRUE,nrow=n)
  data.sim <- as.data.frame(cbind(Y,X))
  
  return(list(data=data.sim, theta = theta, alpha = alpha, sigma = sigma, 
              delta = delta, gamma = gamma, lin_pred = lin_pred, probs = probs))
}


sim_fun2 <- function(model, m, I, k, n, gamma, seed = NULL){
  
  
  if(!is.null(seed)){
    set.seed(seed)
  }
  
  RSM <- GPCM <- FALSE
  if(model %in% c("GRSM","RSM")){
    RSM <- TRUE
  }
  if(model %in% c("GRSM","GPCM","2PL")){
    GPCM <- TRUE
  }
  
  
  q <- k-1
  
  X <- c()
  for(i in 1:m){
    X <- cbind(X, rbinom(n,1,0.5))
  }
  
  
  if(!RSM){
    delta <- deltaX <- matrix(round(rnorm(q*I,sd=0.3),2),nrow=I)
    alpha <- NA
  }else{
    delta <- round(rnorm(I,sd=0.3),2)
    alpha <- round(rnorm(q,sd=0.3),2)
    deltaX <- t(t(matrix(rep(delta,q),nrow=I))+alpha)
  }
  
  
  if(!GPCM){
    sigma <- rep(1,I)
  }else{
    sigma <- seq(0.8,1.2,length=I)
  }
  
  theta <- rnorm(n)
  #
  
  lin_pred<- c()
  probs <- c()
  y <- c()
  
  for(i in 1:n){
    for(ii in 1:I){
      eta <- sigma[ii] * (theta[i] - deltaX[ii,] -sum(gamma[ii,]*X[i,]))
      
      lin_pred <- rbind(lin_pred, eta)
      pi <- responseFun2(eta)
      if(q==1){
        pi <- exp(eta)/(1+exp(eta))
      }
      probs <- rbind(probs,pi)
      pi <- c(pi,1-sum(pi))
      
      # if(any(pi<0)){browser()}
      y.sample <- which(rmultinom(1,1,pi)==1) 
      y <- c(y,y.sample)
    }
  }
  
  Y <- matrix(y,byrow=TRUE,nrow=n)
  data.sim <- as.data.frame(cbind(Y,X))
  
  return(list(data=data.sim, theta = theta, alpha = alpha, sigma = sigma, 
              delta = delta, gamma = gamma, lin_pred = lin_pred, probs = probs))
}


sim_fun3 <- function(model, m, I, k, n, gamma, seed = NULL){
  
  
  if(!is.null(seed)){
    set.seed(seed)
  }
  
  RSM <- GPCM <- FALSE
  if(model %in% c("GRSM","RSM")){
    RSM <- TRUE
  }
  if(model %in% c("GRSM","GPCM","2PL")){
    GPCM <- TRUE
  }
  
  
  q <- k-1
  
  for(i in 1:m){
    if(i%%3 == 1){
      X <- data.frame(V1= rnorm(n))
    }
    if(i%%3 == 2){
      X$V2 <- factor(rbinom(n,1,0.5))
    }
    if(i%%3 == 0){
      X$V3 <- factor(sample(1:4,n,replace=TRUE))
    }
  }
  
  
  X2 <- model.matrix(~V1+V2+V3,data=X)[,-1]
  X2 <- scale(X2)
  
  if(!RSM){
    delta <- deltaX <- matrix(round(rnorm(q*I,sd=0.5,mean=-0.5),2),nrow=I)
    # if(model == "GPCM"){
    #   delta <- deltaX <- delta-0.5
    # }
    alpha <- NA
  }else{
    delta <- round(rnorm(I,sd=0.5),2)
    alpha <- c(0,round(rnorm(q-1,sd=0.5),2))
    deltaX <- t(t(matrix(rep(delta,q),nrow=I))+alpha)
  }
  
  
  if(!GPCM){
    sigma <- rep(1,I)
  }else{
    sigma <- seq(0.7,1,length=I)
  }
  
  theta <- rnorm(n)
  
  lin_pred<- c()
  probs <- c()
  y <- c()
  
  for(i in 1:n){
    for(ii in 1:I){
      eta <- sigma[ii] * (theta[i] - deltaX[ii,] -sum(gamma[ii,]*X2[i,]))
      
      lin_pred <- rbind(lin_pred, eta)
      pi <- responseFun2(eta)
      if(q==1){
        pi <- exp(eta)/(1+exp(eta))
      }
      probs <- rbind(probs,pi)
      pi <- c(pi,1-sum(pi))
      
      # if(any(pi<0)){browser()}
      y.sample <- which(rmultinom(1,1,pi)==1) 
      y <- c(y,y.sample)
    }
  }
  
  Y <- matrix(y,byrow=TRUE,nrow=n)
  data.sim <- as.data.frame(cbind(Y,X))
  names(data.sim)[1:I] <- paste0("Item",1:I)
  
  return(list(data=data.sim, theta = theta, alpha = alpha, sigma = sigma, 
              delta = delta, gamma = gamma, lin_pred = lin_pred, probs = probs))
}



sim_cor <- function(model, m, I, k, n, gamma, sigma, seed = NULL){
  
  
  if(!is.null(seed)){
    set.seed(seed)
  }
  
  RSM <- GPCM <- FALSE
  if(model %in% c("GRSM","RSM")){
    RSM <- TRUE
  }
  if(model %in% c("GRSM","GPCM","2PL")){
    GPCM <- TRUE
  }
  
  
  q <- k-1
  
  mat1 <- rmvnorm(n, sigma = sigma)
  
  set.seed(1860)
  
  X <- c()

  for(i in 1:m){
    if(i%%2 == 1){
      X <- cbind(X, mat1[,i])
    }else{
      X <- cbind(X, mat1[,i]>0)
    }
  }
  X <- scale(X)
  
  if(!RSM){
    delta <- deltaX <- matrix(round(rnorm(q*I,sd=0.5),2),nrow=I)
    # if(model == "GPCM"){
    #   delta <- deltaX <- delta-0.5
    # }
    alpha <- NA
  }else{
    delta <- round(rnorm(I,sd=0.5),2)
    alpha <- c(0,round(rnorm(q-1,sd=0.5),2))
    deltaX <- t(t(matrix(rep(delta,q),nrow=I))+alpha)
  }
  
  
  if(!GPCM){
    sigma <- rep(1,I)
  }else{
    sigma <- seq(0.7,1,length=I)
  }
  
  theta <- rnorm(n)
  
  
  lin_pred<- c()
  probs <- c()
  y <- c()
  
  for(i in 1:n){
    for(ii in 1:I){
      eta <- sigma[ii] * (theta[i] - deltaX[ii,] -sum(gamma[ii,]*X[i,]))
      
      lin_pred <- rbind(lin_pred, eta)
      pi <- responseFun2(eta)
      if(q==1){
        pi <- exp(eta)/(1+exp(eta))
      }
      probs <- rbind(probs,pi)
      pi <- c(pi,1-sum(pi))
      
      # if(any(pi<0)){browser()}
      y.sample <- which(rmultinom(1,1,pi)==1) 
      y <- c(y,y.sample)
    }
  }
  
  Y <- matrix(y,byrow=TRUE,nrow=n)
  data.sim <- as.data.frame(cbind(Y,X))
  
  return(list(data=data.sim, theta = theta, alpha = alpha, sigma = sigma, 
              delta = delta, gamma = gamma, lin_pred = lin_pred, probs = probs))
}

change.param <- function(coefficients,design_list){
  
  ## get basic parameters
  m <- design_list$m
  n <- design_list$n
  I <- design_list$I
  n_sigma <- design_list$n_sigma
  q <- design_list$q
  
  ## extract different parameter matrices
  ## basic model parameters, called delta 
  p.delta <- I*q
  if(design_list$RSM){
    p.delta <- I+q
  }
  delta <- coefficients[,1:p.delta,drop=FALSE]
  
  ## DIF parameters, called gamma
  if(design_list$DSF){
    gamma <- coefficients[,(p.delta+1):(p.delta+I*m*q),drop=FALSE]
    start.sigma <- p.delta+I*m*q+1
  }else{
    gamma <- coefficients[,(p.delta+1):(p.delta+I*m),drop=FALSE]
    start.sigma <- p.delta+I*m+1
  }
  
  ## Discrimination parameters, called sigma
  sigma <- coefficients[,start.sigma:(start.sigma+n_sigma-1),drop=FALSE]
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GPCMlasso-package.R
\docType{data}
\name{tenseness}
\alias{tenseness}
\title{Tenseness data from the Freiburg Complaint Checklist}
\format{
A data frame containing data from the Freiburg Complaint Checklist with 1847 observations. 
All items refer to the scale \emph{Tenseness} and are measured on a 5-point Likert scale where low numbers 
correspond to low frequencies or low intensitites of the respective complaint and vice versa. 
\describe{ 
\item{Clammy_hands}{Do you have clammy hands?}
\item{Sweat_attacks}{Do you have sudden attacks of sweating?}
\item{Clumsiness}{Do you notice that you behave clumsy?}
\item{Wavering_hands}{Are your hands wavering frequently, e.g. when lightning a cigarette or when holding a cup?}
\item{Restless_hands}{Do you notice that your hands are restless?}
\item{Restless_feet}{Do you notice that your feet are restless?}
\item{Twitching_eyes}{Do you notice unvoluntary twitching of your eyes?}
\item{Twitching_mouth}{Do you notice unvoluntary twitching of your mouth?}
\item{Gender}{Gender of the person}
\item{Household}{Does the person live alone in a household or together with somebody?}
\item{Income}{Income, categorized to levels from 1 (low income) to 11(high income). For simplicity,
due to the high number of categories income can be treated as a metric variable.}
\item{WestEast}{Is the person from East Germany (former GDR)?}
\item{Abitur}{Does the person have Abitur (A-levels)?}
\item{Age}{Age of the person}
 }
}
\source{
ZPID (2013). PsychData of the Leibniz Institute for Psychology Information ZPID. Trier: Center for Research Data in Psychology.

Fahrenberg, J. (2010). Freiburg Complaint Checklist [Freiburger Beschwerdenliste (FBL)]. Goettingen, Hogrefe.
}
\description{
Data from the Freiburg Complaint Checklist. 
The data contain all 8 items corresponding to the scale \emph{Tenseness} for 2042 participants of the 
standardization sample of the Freiburg Complaint Checklist.
}
\examples{

data(tenseness)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GPCMlasso-package.R
\docType{data}
\name{tenseness_small}
\alias{tenseness_small}
\title{Subset of tenseness data from the Freiburg Complaint Checklist}
\format{
A data frame containing data from the Freiburg Complaint Checklist a subset of 200 observations. 
The complete data set with 1847 observations can be found in \code{\link{tenseness}}.
All items refer to the scale \emph{Tenseness} and are measured on a 5-point Likert scale where low numbers 
correspond to low frequencies or low intensitites of the respective complaint and vice versa. 
\describe{ 
\item{Clammy_hands}{Do you have clammy hands?}
\item{Sweat_attacks}{Do you have sudden attacks of sweating?}
\item{Clumsiness}{Do you notice that you behave clumsy?}
\item{Wavering_hands}{Are your hands wavering frequently, e.g. when lightning a cigarette or when holding a cup?}
\item{Restless_hands}{Do you notice that your hands are restless?}
\item{Gender}{Gender of the person}
\item{Age}{Age of the person}
 }
}
\source{
ZPID (2013). PsychData of the Leibniz Institute for Psychology Information ZPID. Trier: Center for Research Data in Psychology.

Fahrenberg, J. (2010). Freiburg Complaint Checklist [Freiburger Beschwerdenliste (FBL)]. Goettingen, Hogrefe.
}
\description{
Data from the Freiburg Complaint Checklist. 
The data contain 5 items (out of 8) corresponding to the scale \emph{Tenseness} for a subset of 200 participants of the 
standardization sample of the Freiburg Complaint Checklist.
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\seealso{
\code{\link{GPCMlasso}}, \code{\link{ctrl_GPCMlasso}}, \code{\link{trait.posterior}}
}
\keyword{GPCMlasso}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GPCMlasso-package.R
\docType{package}
\name{GPCMlasso-package}
\alias{GPCMlasso-package}
\title{Find DIF in Generalized Partial Credit Models}
\description{
Performs GPCMlasso, a method to identify DIF in Generalized Partial Credit Models. 
A joint parametric model is set up based on an IRT model chosen by the user. Several variables can be considered simultaneously.
For each pair between variable and item, a parametric DIF effect is introduced which
indicates DIF if the respective parameter is selected (estimated to be unequal zero). 
Parameter selection is done using a lasso-type penalization term.
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\references{
Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
}
\seealso{
\code{\link{GPCMlasso}}
}
\author{
\strong{Maintainer}: Gunther Schauberger \email{gunther.schauberger@tum.de}

Authors:
\itemize{
  \item Gunther Schauberger \email{gunther.schauberger@tum.de}
}


Gunther Schauberger\cr \email{gunther.schauberger@tum.de}
}
\keyword{Credit}
\keyword{DIF}
\keyword{DSF}
\keyword{GPCM}
\keyword{GPCMlasso}
\keyword{Partial}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.GPCMlasso.R
\name{print.GPCMlasso}
\alias{print.GPCMlasso}
\title{Print function for GPCMlasso}
\usage{
\method{print}{GPCMlasso}(x, select = c("BIC", "AIC", "cAIC", "cv"), ...)
}
\arguments{
\item{x}{\code{GPCMlasso} object}

\item{select}{Specifies which criterion to use for the optimal model, we recommend the 
default value "BIC". If cross-validation was performed, automatically the optimal
model according to cross-validation is used. Only the parameter estimates from
the chosen optimal model are printed.}

\item{...}{Further print arguments.}
}
\description{
Print function for a \code{GPCMlasso} object. Prints parameters estimates for all model
components for the optimal model chosen by a specific criterion (by default BIC).
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\references{
Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
}
\seealso{
\code{\link{GPCMlasso}}
}
\author{
Gunther Schauberger\cr \email{gunther.schauberger@tum.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.GPCMlasso.R
\name{predict.GPCMlasso}
\alias{predict.GPCMlasso}
\title{Predict function for GPCMlasso}
\usage{
\method{predict}{GPCMlasso}(
  object,
  coefs = NULL,
  newdata = NULL,
  type = c("link", "response"),
  ...
)
}
\arguments{
\item{object}{\code{GPCMlasso} object}

\item{coefs}{Optional vector of coefficients, can be filled with a specific 
row from \code{object$coefficients}. If not specified, \code{coefs} are specififed to be
the BIC-optimal coefficients or, if cross-validation was performed, the optimal 
coefficients according to cross-validation.}

\item{newdata}{List possibly containing slots Y, X, Z1 and Z2 to use new data for prediction.}

\item{type}{Type "link" gives vectors of linear predictors for separate categories 
(of length $k_i-1$) and type "response" gives the respective probabilities (of length $k_i$).}

\item{...}{Further predict arguments.}
}
\description{
Predict function for a \code{GPCMlasso} object. 
Predictions can be linear predictors or probabilities separately for each person and each item.
}
\details{
Results are lists of vectors with length equal to the number 
of response categories $k_i$ in case of
probabilities (\code{type="response"}) or 
$k_i-1$ in case of linear predictors (\code{type="link"}).
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\seealso{
\code{\link{GPCMlasso}}
}
\author{
Gunther Schauberger\cr \email{gunther@stat.uni-muenchen.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.GPCMlasso.R
\name{plot.GPCMlasso}
\alias{plot.GPCMlasso}
\title{Plot function for GPCMlasso}
\usage{
\method{plot}{GPCMlasso}(x, select = c("BIC", "AIC", "cAIC", "cv"), 
type = c("DIF", "Variable"),
log.lambda = TRUE, items_per_page = 1, items = "all", 
columns = NULL, ask_new = TRUE, lambda.lines = TRUE,
equal_range = TRUE, add.to.lambda = 0, \dots)
}
\arguments{
\item{x}{\code{GPCMlasso} object}

\item{select}{Specifies which criterion to use for the optimal model, we recommend the 
default value "BIC". If cross-validation was performed, automatically the optimal
model according to cross-validation is used. The chosen optimal model is 
highlighted with a red dashed line.}

\item{type}{Which plot type should be used. \code{DIF} means that one plot per item is shown, 
indicating whether DIF was found for this item. \code{Variable} means that
one plot per variable is shown, which is useful for the option \code{fuse.per.variable}.}

\item{log.lambda}{A logical value indicating whether lambda or a log-transformation of 
lambda should be used as x-axis in the plots.}

\item{items_per_page}{By default, each plot/item is put on a separate page. For example,
\code{items_per_page=4} would put four plots/items on one page.}

\item{items}{By default, all items are plotted. If \code{items=c(1,3)}, 
only the first and the third item are plotted.}

\item{columns}{Specifies the number of columns to use when several
plots are on one page. Only relevant if \code{items_per_page}>1.}

\item{ask_new}{If TRUE, the user is asked to confirm before the next item is plotted.}

\item{lambda.lines}{A logical value indicating whether a thin gray line plotted
for each value from the vector of tuning parameters from \code{object}}

\item{equal_range}{A logical value indicating whether for each plot equal limits 
on the y-axis shall be used.}

\item{add.to.lambda}{Constant c to be added to lambda as in log(lambda + c) for plotting coefficient paths.}

\item{...}{Further plot arguments.}
}
\description{
Plot function for a \code{GPCMlasso} object. Plots show coefficient paths
of DIF (or DSF) parameters along (a transformation of) the tuning parameter lambda.
One plot per item is created, every single parameter corresponding to this item
is depicted by a single path. 
The optimal model is highlighted with a red dashed line.
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\references{
Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
}
\seealso{
\code{\link{GPCMlasso}}
}
\author{
Gunther Schauberger\cr \email{gunther.schauberger@tum.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GPCMlasso.R
\name{GPCMlasso}
\alias{GPCMlasso}
\title{GPCMlasso}
\usage{
GPCMlasso(
  formula,
  data,
  DSF = FALSE,
  model = c("PCM", "RSM", "GPCM", "GRSM", "RM", "2PL"),
  control = ctrl_GPCMlasso(),
  cv = FALSE,
  main.effects = TRUE
)
}
\arguments{
\item{formula}{Formula to indicate which items are considered and which covariates 
should be used to find DIF. Items are considered to be the response and are 
concatenated by \code{cbind()}. If the RHS of the formula is \code{~0}, simply the 
model specified in \code{model} is calulated.}

\item{data}{Data frame containing the ordinal item response data 
(as ordered factors) and all covariates.}

\item{DSF}{Should Differential Step Functioning (DSF) be considered? If \code{DSF = TRUE}, one parameter per step between two response categories is introduced. For binary items, DSF and DIF conincide.}

\item{model}{Specify the underlying basic model. Currently, you can choose 
between the partial credit model and the rating scale model and
the respective generalized versions of both models called 'PCM', 'RSM', 
'GPCM' and 'GRSM'. Generalized models allow for different discrimination 
parameters between items.}

\item{control}{Control argument to specify further arguments for the algorithm
and numerical optimization, specified by \code{\link{ctrl_GPCMlasso}}.}

\item{cv}{Should cross-validation be performed? Cross-validation can be used as an alternative to BIC to select the optimal tuning parameter.}

\item{main.effects}{Should also main effects of the variables be included in the model? Default is \code{TRUE}. Here, positive parameter estimates correspond 
to an increase of the respective trait if the variable increases.}
}
\value{
\item{coefficients}{Matrix containing all parameters for the GPCMlasso model, one row
per tuning parameter lambda. Due to the penalty the parameters are scaled and, therefore,
are comparable with respect to their size.} 
\item{logLik}{Vector of log-likelihoods, one value per tuning parameter lambda.}
\item{call}{The function call of \code{GPCMlasso}}
\item{cv_error}{Vector of cv_errors, one per tuning parameter. Only relevant if \code{cv = TRUE}.}
\item{model}{Basic IRT model chosen by user.}
\item{data}{Data from call.}
\item{control}{Control list.}
\item{DSF}{DSF from call.}
\item{formula}{Formula from call.}
\item{item.names}{Item names.}
\item{Y}{Matrix containing item responses.}
\item{design_list}{List containing several helpful objects for internal use.}
\item{AIC}{Vector of AIC values, one per tuning parameter.}
\item{BIC}{Vector of BIC values, one per tuning parameter.}
\item{cAIC}{Vector of corrected AIC values, one per tuning parameter.}
\item{df}{Vector of degrees of freedom, one per tuning parameter.}
\item{coef.rescal}{Matrix containing all rescaled parameters for the GPCMlasso model, one row
per tuning parameter lambda. In contrast to \code{coefficients}, all parameters are rescaled back
to their original scales.}
}
\description{
Performs GPCMlasso, a method to identify differential item functioning (DIF) in Generalized Partial Credit Models. 
A joint parametric model is set up based on an IRT model chosen by the user. Several variables can be considered simultaneously.
For each pair between variable and item, a parametric DIF effect is introduced which
indicates DIF if the respective parameter is selected (estimated to be unequal zero). 
Parameter selection is done using a lasso-type penalization term.
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\references{
Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
}
\seealso{
\code{\link{GPCMlasso-package}}, \code{\link{ctrl_GPCMlasso}}, \code{\link{print.GPCMlasso}}, 
\code{\link{plot.GPCMlasso}}, \code{\link{predict.GPCMlasso}}, \code{\link{trait.posterior}}
}
\author{
Gunther Schauberger\cr \email{gunther.schauberger@tum.de}
}
\keyword{GPCMlasso}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctrl_GPCMlasso.R
\name{ctrl_GPCMlasso}
\alias{ctrl_GPCMlasso}
\title{Control function for GPCMlasso}
\usage{
ctrl_GPCMlasso(
  log.lambda = TRUE,
  lambda = NULL,
  l.lambda = 50,
  lambda.min = 0.1,
  penalize.main.effects = FALSE,
  fuse.per.variable = FALSE,
  adaptive = TRUE,
  weight.penalties = TRUE,
  ada.lambda = 1e-04,
  ada.power = 1,
  Q = 15,
  lambda2 = 1e-04,
  cvalue = 1e-05,
  trace = TRUE,
  folds = 10,
  cores = 25,
  null_thresh = 0.01,
  gradtol = 1e-06,
  steptol = 1e-06,
  iterlim = 500,
  precision = 3,
  all.dummies = FALSE,
  ctrl.gpcm = list()
)
}
\arguments{
\item{log.lambda}{Should the grid of tuning parameters be created on a log scale?}

\item{lambda}{Optional argument to specify a vector of tuning parameters. If \code{lambda = NULL}, a vector of length \code{l.lambda} is created automatically.}

\item{l.lambda}{Specifies the length of the grid of tuning parameters.}

\item{lambda.min}{Minimal value used if the grid of tuning parameters is created automatically.}

\item{penalize.main.effects}{Should also main covariate effects be penalized? Default is \code{FALSE}.}

\item{fuse.per.variable}{Should fusion be applied per variable? This option 
creates clusters of items with equal effects for a variable.}

\item{adaptive}{Should adaptive lasso be used? Default is \code{TRUE}.}

\item{weight.penalties}{Should penalties be weighted according to the number of penalty term and the number of parameters
corresponding to one pair between item and covariate. Only relevant if both \code{DSF = TRUE} and the number of 
response categories differs across items (because only then these values can differ).}

\item{ada.lambda}{Size of tuning parameter for Ridge-regularized estimation of parameters used for adaptive weights.}

\item{ada.power}{By default, 1st power of absolute values of Ridge-regularized estimates are used. Could be changed to squared values by \code{ada-power = 2}.}

\item{Q}{Number of nodes to be used in Gauss-Hermite quadrature.}

\item{lambda2}{Tuning parameter for ridge penalty on all coefficients except 
sigma/slope parameters.
Should be small, only used to stabilize results.}

\item{cvalue}{Internal parameter for the quadratic approximation of the L1
penalty. Should be sufficiently small.}

\item{trace}{Should the trace of the progress (current tuning parameter) be printed?}

\item{folds}{Number of folds for cross-validation. Only relevant if \code{cv = TRUE} in \code{\link{GPCMlasso}}.}

\item{cores}{Number of cores to be used parallel when fitting the model.}

\item{null_thresh}{Threshold which is used to distinguih between values equal and unequal to zero.}

\item{gradtol}{Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.}

\item{steptol}{Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.}

\item{iterlim}{Parameter to tune optimization accuracy, for details see \code{\link{nlm}}.}

\item{precision}{Number of decimal places used to round coefficient estimates.}

\item{all.dummies}{Should (in case of factors with more than 2 categories) the dummy variables for all categories be included in the design matrix? If \code{all.dummies = TRUE}, the dependence on the reference category is eliminated for multi-categorical covariates.}

\item{ctrl.gpcm}{List of control arguments for initial \code{\link[ltm]{gpcm}} fit, which is needed to get good starting values. Does not apply to RSM or GRSM.}
}
\description{
Control parameters for penalty terms and for tuning the fitting algorithm.
}
\examples{
data(tenseness_small)

## formula for simple model without covariates
form.0 <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~0"))

######
## fit simple RSM where loglikelihood and score function are evaluated parallel on 2 cores
rsm.0 <- GPCMlasso(form.0, tenseness_small, model = "RSM", 
control= ctrl_GPCMlasso(cores=2))
rsm.0

\dontrun{
## formula for model with covariates (and DIF detection)
form <- as.formula(paste("cbind(",paste(colnames(tenseness_small)[1:5],collapse=","),")~."))

######
## fit GPCM model with 10 different tuning parameters
gpcm <- GPCMlasso(form, tenseness_small, model = "GPCM", 
                  control = ctrl_GPCMlasso(l.lambda = 10))
gpcm
plot(gpcm)
pred.gpcm <- predict(gpcm)
trait.gpcm <- trait.posterior(gpcm)

######
## fit RSM, detect differential step functioning (DSF)
rsm.DSF <- GPCMlasso(form, tenseness_small, model = "RSM", DSF = TRUE, 
                     control = ctrl_GPCMlasso(l.lambda = 10))
rsm.DSF
plot(rsm.DSF)

## create binary data set
tenseness_small_binary <- tenseness_small
tenseness_small_binary[,1:5][tenseness_small[,1:5]>1] <- 2

######
## fit and cross-validate Rasch model
set.seed(1860)
rm.cv <- GPCMlasso(form, tenseness_small_binary, model = "RM", cv = TRUE, 
                   control = ctrl_GPCMlasso(l.lambda = 10))
rm.cv
plot(rm.cv)
}
}
\references{
Schauberger, Gunther and Mair, Patrick (2019): A Regularization Approach for the Detection of Differential 
Item Functioning in Generalized Partial Credit Models, \emph{Behavior Research Methods}, \url{https://link.springer.com/article/10.3758/s13428-019-01224-2}
}
\author{
Gunther Schauberger\cr \email{gunther.schauberger@tum.de}
}
